# -*- coding: utf-8 -*-
"""
Integration tests for the docker_network states
"""
# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import errno
import functools
import logging
import os
import subprocess
import tempfile

# Import Salt Libs
import salt.utils.files
import salt.utils.network
import salt.utils.path
from salt.exceptions import CommandExecutionError
from tests.support.case import ModuleCase
from tests.support.docker import random_name, with_network
from tests.support.helpers import destructiveTest, requires_system_grains, slowTest
from tests.support.mixins import SaltReturnAssertsMixin

# Import Salt Testing Libs
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import skipIf

log = logging.getLogger(__name__)


IMAGE_NAME = random_name(prefix="salt_busybox_")
IPV6_ENABLED = bool(salt.utils.network.ip_addrs6(include_loopback=True))


def network_name(func):
    """
    Generate a randomized name for a network and clean it up afterward
    """

    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        name = random_name(prefix="salt_net_")
        try:
            return func(self, name, *args, **kwargs)
        finally:
            self.run_function("docker.disconnect_all_containers_from_network", [name])
            try:
                self.run_function("docker.remove_network", [name])
            except CommandExecutionError as exc:
                if "No such network" not in exc.__str__():
                    raise

    return wrapper


def container_name(func):
    """
    Generate a randomized name for a container and clean it up afterward
    """

    def build_image():
        # Create temp dir
        image_build_rootdir = tempfile.mkdtemp(dir=RUNTIME_VARS.TMP)
        script_path = os.path.join(RUNTIME_VARS.BASE_FILES, "mkimage-busybox-static")
        cmd = [script_path, image_build_rootdir, IMAGE_NAME]
        log.debug("Running '%s' to build busybox image", " ".join(cmd))
        process = subprocess.Popen(
            cmd, close_fds=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )
        output = process.communicate()[0]
        log.debug("Output from mkimge-busybox-static:\n%s", output)

        if process.returncode != 0:
            raise Exception("Failed to build image")

        try:
            salt.utils.files.rm_rf(image_build_rootdir)
        except OSError as exc:
            if exc.errno != errno.ENOENT:
                raise

    @functools.wraps(func)
    def wrapper(self, *args, **kwargs):
        try:
            self.run_function("docker.inspect_image", [IMAGE_NAME])
        except CommandExecutionError:
            pass
        else:
            build_image()

        name = random_name(prefix="salt_test_")
        self.run_function(
            "docker.create",
            name=name,
            image=IMAGE_NAME,
            command="sleep 600",
            start=True,
        )
        try:
            return func(self, name, *args, **kwargs)
        finally:
            try:
                self.run_function("docker.rm", [name], force=True)
            except CommandExecutionError as exc:
                if "No such container" not in exc.__str__():
                    raise

    return wrapper


@slowTest
@destructiveTest
@skipIf(not salt.utils.path.which("dockerd"), "Docker not installed")
class DockerNetworkTestCase(ModuleCase, SaltReturnAssertsMixin):
    """
    Test docker_network states
    """

    @classmethod
    def tearDownClass(cls):
        """
        Remove test image if present. Note that this will run a docker rmi even
        if no test which required the image was run.
        """
        cmd = ["docker", "rmi", "--force", IMAGE_NAME]
        log.debug("Running '%s' to destroy busybox image", " ".join(cmd))
        process = subprocess.Popen(
            cmd, close_fds=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT
        )
        output = process.communicate()[0]
        log.debug("Output from %s:\n%s", " ".join(cmd), output)

        if process.returncode != 0 and "No such image" not in str(output):
            raise Exception("Failed to destroy image")

    def run_state(self, function, **kwargs):
        ret = super(DockerNetworkTestCase, self).run_state(function, **kwargs)
        log.debug("ret = %s", ret)
        return ret

    @with_network(create=False)
    @slowTest
    def test_absent(self, net):
        self.assertSaltTrueReturn(
            self.run_state("docker_network.present", name=net.name)
        )
        ret = self.run_state("docker_network.absent", name=net.name)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        self.assertEqual(ret["changes"], {"removed": True})
        self.assertEqual(ret["comment"], "Removed network '{0}'".format(net.name))

    @container_name
    @with_network(create=False)
    @slowTest
    def test_absent_with_disconnected_container(self, net, container_name):
        self.assertSaltTrueReturn(
            self.run_state(
                "docker_network.present", name=net.name, containers=[container_name]
            )
        )
        ret = self.run_state("docker_network.absent", name=net.name)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        self.assertEqual(
            ret["changes"], {"removed": True, "disconnected": [container_name]}
        )
        self.assertEqual(ret["comment"], "Removed network '{0}'".format(net.name))

    @with_network(create=False)
    @slowTest
    def test_absent_when_not_present(self, net):
        ret = self.run_state("docker_network.absent", name=net.name)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]
        self.assertEqual(ret["changes"], {})
        self.assertEqual(
            ret["comment"], "Network '{0}' already absent".format(net.name)
        )

    @with_network(create=False)
    @slowTest
    def test_present(self, net):
        ret = self.run_state("docker_network.present", name=net.name)
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        # Make sure the state return is what we expect
        self.assertEqual(ret["changes"], {"created": True})
        self.assertEqual(ret["comment"], "Network '{0}' created".format(net.name))

        # Now check to see that the network actually exists. If it doesn't,
        # this next function call will raise an exception.
        self.run_function("docker.inspect_network", [net.name])

    @container_name
    @with_network(create=False)
    @slowTest
    def test_present_with_containers(self, net, container_name):
        ret = self.run_state(
            "docker_network.present", name=net.name, containers=[container_name]
        )
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        self.assertEqual(
            ret["changes"], {"created": True, "connected": [container_name]}
        )
        self.assertEqual(ret["comment"], "Network '{0}' created".format(net.name))

        # Now check to see that the network actually exists. If it doesn't,
        # this next function call will raise an exception.
        self.run_function("docker.inspect_network", [net.name])

    def _test_present_reconnect(self, net, container_name, reconnect=True):
        ret = self.run_state("docker_network.present", name=net.name, driver="bridge")
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        self.assertEqual(ret["changes"], {"created": True})
        self.assertEqual(ret["comment"], "Network '{0}' created".format(net.name))

        # Connect the container
        self.run_function(
            "docker.connect_container_to_network", [container_name, net.name]
        )

        # Change the driver to force the network to be replaced
        ret = self.run_state(
            "docker_network.present",
            name=net.name,
            driver="macvlan",
            reconnect=reconnect,
        )
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        self.assertEqual(
            ret["changes"],
            {
                "recreated": True,
                "reconnected" if reconnect else "disconnected": [container_name],
                net.name: {"Driver": {"old": "bridge", "new": "macvlan"}},
            },
        )
        self.assertEqual(
            ret["comment"],
            "Network '{0}' was replaced with updated config".format(net.name),
        )

    @container_name
    @with_network(create=False)
    @slowTest
    def test_present_with_reconnect(self, net, container_name):
        """
        Test reconnecting with containers not passed to state
        """
        self._test_present_reconnect(net, container_name, reconnect=True)

    @container_name
    @with_network(create=False)
    @slowTest
    def test_present_with_no_reconnect(self, net, container_name):
        """
        Test reconnecting with containers not passed to state
        """
        self._test_present_reconnect(net, container_name, reconnect=False)

    @with_network()
    @slowTest
    def test_present_internal(self, net):
        self.assertSaltTrueReturn(
            self.run_state("docker_network.present", name=net.name, internal=True,)
        )
        net_info = self.run_function("docker.inspect_network", [net.name])
        self.assertIs(net_info["Internal"], True)

    @with_network()
    @slowTest
    def test_present_labels(self, net):
        # Test a mix of different ways of specifying labels
        self.assertSaltTrueReturn(
            self.run_state(
                "docker_network.present",
                name=net.name,
                labels=["foo", "bar=baz", {"hello": "world"}],
            )
        )
        net_info = self.run_function("docker.inspect_network", [net.name])
        self.assertEqual(
            net_info["Labels"], {"foo": "", "bar": "baz", "hello": "world"},
        )

    @with_network(subnet="fe3f:2180:26:1::/123")
    @with_network(subnet="10.247.197.96/27")
    @skipIf(not IPV6_ENABLED, "IPv6 not enabled")
    @slowTest
    def test_present_enable_ipv6(self, net1, net2):
        self.assertSaltTrueReturn(
            self.run_state(
                "docker_network.present",
                name=net1.name,
                enable_ipv6=True,
                ipam_pools=[{"subnet": net1.subnet}, {"subnet": net2.subnet}],
            )
        )
        net_info = self.run_function("docker.inspect_network", [net1.name])
        self.assertIs(net_info["EnableIPv6"], True)

    @requires_system_grains
    @with_network()
    @slowTest
    def test_present_attachable(self, net, grains):
        if grains["os_family"] == "RedHat" and grains.get("osmajorrelease", 0) <= 7:
            self.skipTest("Cannot reliably manage attachable on RHEL <= 7")

        self.assertSaltTrueReturn(
            self.run_state("docker_network.present", name=net.name, attachable=True,)
        )
        net_info = self.run_function("docker.inspect_network", [net.name])
        self.assertIs(net_info["Attachable"], True)

    @skipIf(True, "Skip until we can set up docker swarm testing")
    @with_network()
    def test_present_scope(self, net):
        self.assertSaltTrueReturn(
            self.run_state("docker_network.present", name=net.name, scope="global",)
        )
        net_info = self.run_function("docker.inspect_network", [net.name])
        self.assertIs(net_info["Scope"], "global")

    @skipIf(True, "Skip until we can set up docker swarm testing")
    @with_network()
    def test_present_ingress(self, net):
        self.assertSaltTrueReturn(
            self.run_state("docker_network.present", name=net.name, ingress=True,)
        )
        net_info = self.run_function("docker.inspect_network", [net.name])
        self.assertIs(net_info["Ingress"], True)

    @with_network(subnet="10.247.197.128/27")
    @with_network(subnet="10.247.197.96/27")
    @slowTest
    def test_present_with_custom_ipv4(self, net1, net2):
        # First run will test passing the IPAM arguments individually
        self.assertSaltTrueReturn(
            self.run_state(
                "docker_network.present",
                name=net1.name,
                subnet=net1.subnet,
                gateway=net1.gateway,
            )
        )
        # Second run will pass them in the ipam_pools argument
        ret = self.run_state(
            "docker_network.present",
            name=net1.name,  # We want to keep the same network name
            ipam_pools=[{"subnet": net2.subnet, "gateway": net2.gateway}],
        )
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        # Docker requires there to be IPv4, even when only an IPv6 subnet was
        # provided. So, there will be both an IPv4 and IPv6 pool in the
        # configuration.
        expected = {
            "recreated": True,
            net1.name: {
                "IPAM": {
                    "Config": {
                        "old": [{"Subnet": net1.subnet, "Gateway": net1.gateway}],
                        "new": [{"Subnet": net2.subnet, "Gateway": net2.gateway}],
                    }
                }
            },
        }
        self.assertEqual(ret["changes"], expected)
        self.assertEqual(
            ret["comment"],
            "Network '{0}' was replaced with updated config".format(net1.name),
        )

    @with_network(subnet="fe3f:2180:26:1::20/123")
    @with_network(subnet="fe3f:2180:26:1::/123")
    @with_network(subnet="10.247.197.96/27")
    @skipIf(not IPV6_ENABLED, "IPv6 not enabled")
    @slowTest
    def test_present_with_custom_ipv6(self, ipv4_net, ipv6_net1, ipv6_net2):
        self.assertSaltTrueReturn(
            self.run_state(
                "docker_network.present",
                name=ipv4_net.name,
                enable_ipv6=True,
                ipam_pools=[
                    {"subnet": ipv4_net.subnet, "gateway": ipv4_net.gateway},
                    {"subnet": ipv6_net1.subnet, "gateway": ipv6_net1.gateway},
                ],
            )
        )

        ret = self.run_state(
            "docker_network.present",
            name=ipv4_net.name,  # We want to keep the same network name
            enable_ipv6=True,
            ipam_pools=[
                {"subnet": ipv4_net.subnet, "gateway": ipv4_net.gateway},
                {"subnet": ipv6_net2.subnet, "gateway": ipv6_net2.gateway},
            ],
        )
        self.assertSaltTrueReturn(ret)
        ret = ret[next(iter(ret))]

        # Docker requires there to be IPv4, even when only an IPv6 subnet was
        # provided. So, there will be both an IPv4 and IPv6 pool in the
        # configuration.
        expected = {
            "recreated": True,
            ipv4_net.name: {
                "IPAM": {
                    "Config": {
                        "old": [
                            {"Subnet": ipv4_net.subnet, "Gateway": ipv4_net.gateway},
                            {"Subnet": ipv6_net1.subnet, "Gateway": ipv6_net1.gateway},
                        ],
                        "new": [
                            {"Subnet": ipv4_net.subnet, "Gateway": ipv4_net.gateway},
                            {"Subnet": ipv6_net2.subnet, "Gateway": ipv6_net2.gateway},
                        ],
                    }
                }
            },
        }
        self.assertEqual(ret["changes"], expected)
        self.assertEqual(
            ret["comment"],
            "Network '{0}' was replaced with updated config".format(ipv4_net.name),
        )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Tyler Johnson <tjohnson@saltstack.com>
"""

# Import Salt Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
from salt.cloud.clouds import proxmox

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import ANY, MagicMock, patch
from tests.support.unit import TestCase


class ProxmoxTest(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {
            proxmox: {
                "__utils__": {
                    "cloud.fire_event": MagicMock(),
                    "cloud.filter_event": MagicMock(),
                    "cloud.bootstrap": MagicMock(),
                },
                "__opts__": {
                    "sock_dir": True,
                    "transport": True,
                    "providers": {"my_proxmox": {}},
                    "profiles": {"my_proxmox": {}},
                },
                "__active_provider_name__": "my_proxmox:proxmox",
            }
        }

    def setUp(self):
        self.vm_ = {
            "profile": "my_proxmox",
            "name": "vm4",
            "driver": "proxmox",
            "technology": "qemu",
            "host": "127.0.0.1",
            "clone": True,
            "ide0": "data",
            "sata0": "data",
            "scsi0": "data",
            "net0": "a=b,c=d",
        }

    def tearDown(self):
        del self.vm_

    def test__stringlist_to_dictionary(self):
        result = proxmox._stringlist_to_dictionary("")
        self.assertEqual(result, {})

        result = proxmox._stringlist_to_dictionary(
            "foo=bar, ignored_space=bar,internal space=bar"
        )
        self.assertEqual(
            result, {"foo": "bar", "ignored_space": "bar", "internal space": "bar"}
        )

        # Negative cases
        self.assertRaises(ValueError, proxmox._stringlist_to_dictionary, "foo=bar,foo")
        self.assertRaises(
            ValueError,
            proxmox._stringlist_to_dictionary,
            "foo=bar,totally=invalid=assignment",
        )

    def test__dictionary_to_stringlist(self):
        result = proxmox._dictionary_to_stringlist({})
        self.assertEqual(result, "")

        result = proxmox._dictionary_to_stringlist({"a": "a"})
        self.assertEqual(result, "a=a")

        result = proxmox._dictionary_to_stringlist({"a": "a", "b": "b"})
        self.assertEqual(result, "a=a,b=b")

    def test__reconfigure_clone(self):
        # The return_value is for the net reconfigure assertions, it is irrelevant for the rest
        with patch.object(
            proxmox, "query", return_value={"net0": "c=overwritten,g=h"}
        ) as query:
            # Test a vm that lacks the required attributes
            proxmox._reconfigure_clone({}, 0)
            query.assert_not_called()

            # Test a fully mocked vm
            proxmox._reconfigure_clone(self.vm_, 0)

            # net reconfigure
            query.assert_any_call("get", "nodes/127.0.0.1/qemu/0/config")
            query.assert_any_call(
                "post", "nodes/127.0.0.1/qemu/0/config", {"net0": "a=b,c=d,g=h"}
            )

            # hdd reconfigure
            query.assert_any_call(
                "post", "nodes/127.0.0.1/qemu/0/config", {"ide0": "data"}
            )
            query.assert_any_call(
                "post", "nodes/127.0.0.1/qemu/0/config", {"sata0": "data"}
            )
            query.assert_any_call(
                "post", "nodes/127.0.0.1/qemu/0/config", {"scsi0": "data"}
            )

    def test_clone(self):
        """
        Test that an integer value for clone_from
        """
        mock_query = MagicMock(return_value="")
        with patch(
            "salt.cloud.clouds.proxmox._get_properties", MagicMock(return_value=[])
        ), patch("salt.cloud.clouds.proxmox.query", mock_query):
            vm_ = {
                "technology": "qemu",
                "name": "new2",
                "host": "myhost",
                "clone": True,
                "clone_from": 123,
            }

            # CASE 1: Numeric ID
            result = proxmox.create_node(vm_, ANY)
            mock_query.assert_called_once_with(
                "post", "nodes/myhost/qemu/123/clone", {"newid": ANY},
            )
            assert result == {}

            # CASE 2: host:ID notation
            mock_query.reset_mock()
            vm_["clone_from"] = "otherhost:123"
            result = proxmox.create_node(vm_, ANY)
            mock_query.assert_called_once_with(
                "post", "nodes/otherhost/qemu/123/clone", {"newid": ANY},
            )
            assert result == {}



# coding: utf-8

# Python libs
from __future__ import absolute_import

from collections import namedtuple

# Salt libs
import salt.beacons.memusage as memusage
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch

# Salt testing libs
from tests.support.unit import TestCase

STUB_MEMORY_USAGE = namedtuple(
    "vmem", "total available percent used free active inactive buffers cached shared"
)(
    15722012672,
    9329594368,
    40.7,
    5137018880,
    4678086656,
    6991405056,
    2078953472,
    1156378624,
    4750528512,
    898908160,
)


class MemUsageBeaconTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test case for salt.beacons.memusage
    """

    def setup_loader_modules(self):
        return {}

    def test_non_list_config(self):
        config = {}

        ret = memusage.validate(config)

        self.assertEqual(
            ret, (False, "Configuration for memusage beacon must be a list.")
        )

    def test_empty_config(self):
        config = [{}]

        ret = memusage.validate(config)

        self.assertEqual(
            ret, (False, "Configuration for memusage beacon requires percent.")
        )

    def test_memusage_match(self):
        with patch("psutil.virtual_memory", MagicMock(return_value=STUB_MEMORY_USAGE)):

            config = [{"percent": "40%"}, {"interval": 30}]

            ret = memusage.validate(config)

            self.assertEqual(ret, (True, "Valid beacon configuration"))

            ret = memusage.beacon(config)
            self.assertEqual(ret, [{"memusage": 40.7}])

    def test_memusage_nomatch(self):
        with patch("psutil.virtual_memory", MagicMock(return_value=STUB_MEMORY_USAGE)):

            config = [{"percent": "70%"}]

            ret = memusage.validate(config)

            self.assertEqual(ret, (True, "Valid beacon configuration"))

            ret = memusage.beacon(config)
            self.assertNotEqual(ret, [{"memusage": 50}])



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.ini_manage as ini_manage

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase

# pylint: disable=no-member


class IniManageTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.ini_manage
    """

    def setup_loader_modules(self):
        return {ini_manage: {}}

    # 'options_present' function tests: 1

    def test_options_present(self):
        """
        Test to verify options present in file.
        """
        name = "salt"

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        with patch.dict(ini_manage.__opts__, {"test": True}):
            comt = ""
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(ini_manage.options_present(name), ret)

        changes = {
            "first": "who is on",
            "second": "what is on",
            "third": "I don't know",
        }
        with patch.dict(
            ini_manage.__salt__, {"ini.set_option": MagicMock(return_value=changes)}
        ):
            with patch.dict(ini_manage.__opts__, {"test": False}):
                comt = "Changes take effect"
                ret.update({"comment": comt, "result": True, "changes": changes})
                self.assertDictEqual(ini_manage.options_present(name), ret)

        original = {
            "mysection": {
                "first": "who is on",
                "second": "what is on",
                "third": "I don't know",
            }
        }
        desired = {"mysection": {"first": "who is on", "second": "what is on"}}
        changes = {
            "mysection": {
                "first": "who is on",
                "second": "what is on",
                "third": {"after": None, "before": "I don't know"},
            }
        }
        with patch.dict(
            ini_manage.__salt__, {"ini.get_ini": MagicMock(return_value=original)}
        ):
            with patch.dict(
                ini_manage.__salt__,
                {"ini.remove_option": MagicMock(return_value="third")},
            ):
                with patch.dict(
                    ini_manage.__salt__,
                    {"ini.get_option": MagicMock(return_value="I don't know")},
                ):
                    with patch.dict(
                        ini_manage.__salt__,
                        {"ini.set_option": MagicMock(return_value=desired)},
                    ):
                        with patch.dict(ini_manage.__opts__, {"test": False}):
                            comt = "Changes take effect"
                            ret.update(
                                {"comment": comt, "result": True, "changes": changes}
                            )
                            self.assertDictEqual(
                                ini_manage.options_present(name, desired, strict=True),
                                ret,
                            )

    # 'options_absent' function tests: 1

    def test_options_absent(self):
        """
        Test to verify options absent in file.
        """
        name = "salt"

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        with patch.dict(ini_manage.__opts__, {"test": True}):
            comt = "No changes detected."
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(ini_manage.options_absent(name), ret)

        with patch.dict(ini_manage.__opts__, {"test": False}):
            comt = "No anomaly detected"
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(ini_manage.options_absent(name), ret)
        original = {"Tables": {"key1": "1", "key2": "2", "key3": "3", "key4": "4"}}
        sections = {"Tables": ["key2", "key3"]}
        changes = {"Tables": {"key2": "2", "key3": "3"}}
        with patch.dict(
            ini_manage.__salt__,
            {"ini.remove_option": MagicMock(side_effect=["2", "3"])},
        ):
            with patch.dict(ini_manage.__opts__, {"test": False}):
                comt = "Changes take effect"
                ret.update({"comment": comt, "result": True, "changes": changes})
                self.assertDictEqual(ini_manage.options_absent(name, sections), ret)

    # 'sections_present' function tests: 1

    def test_sections_present(self):
        """
        Test to verify sections present in file.
        """
        name = "salt"

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        with patch.dict(ini_manage.__opts__, {"test": True}):
            with patch.dict(
                ini_manage.__salt__, {"ini.get_ini": MagicMock(return_value=None)}
            ):
                comt = "No changes detected."
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(ini_manage.sections_present(name), ret)

        changes = {
            "first": "who is on",
            "second": "what is on",
            "third": "I don't know",
        }
        with patch.dict(
            ini_manage.__salt__, {"ini.set_option": MagicMock(return_value=changes)}
        ):
            with patch.dict(ini_manage.__opts__, {"test": False}):
                comt = "Changes take effect"
                ret.update({"comment": comt, "result": True, "changes": changes})
                self.assertDictEqual(ini_manage.sections_present(name), ret)

    # 'sections_absent' function tests: 1

    def test_sections_absent(self):
        """
        Test to verify sections absent in file.
        """
        name = "salt"

        ret = {"name": name, "result": None, "comment": "", "changes": {}}

        with patch.dict(ini_manage.__opts__, {"test": True}):
            with patch.dict(
                ini_manage.__salt__, {"ini.get_ini": MagicMock(return_value=None)}
            ):
                comt = "No changes detected."
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(ini_manage.sections_absent(name), ret)

        with patch.dict(ini_manage.__opts__, {"test": False}):
            comt = "No anomaly detected"
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(ini_manage.sections_absent(name), ret)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt libs
import salt.utils.dictdiffer as dictdiffer

# Import Salt Testing libs
from tests.support.unit import TestCase

NONE = dictdiffer.RecursiveDictDiffer.NONE_VALUE


class RecursiveDictDifferTestCase(TestCase):
    def setUp(self):
        old_dict = {
            "a": {"b": 1, "c": 2, "e": "old_value", "f": "old_key"},
            "j": "value",
        }
        new_dict = {
            "a": {"b": 1, "c": 4, "e": "new_value", "g": "new_key"},
            "h": "new_key",
            "i": None,
            "j": "value",
        }
        self.recursive_diff = dictdiffer.recursive_diff(
            old_dict, new_dict, ignore_missing_keys=False
        )
        self.recursive_diff_ign = dictdiffer.recursive_diff(old_dict, new_dict)

    def tearDown(self):
        for attrname in ("recursive_diff", "recursive_diff_missing_keys"):
            try:
                delattr(self, attrname)
            except AttributeError:
                continue

    def test_added(self):
        self.assertEqual(self.recursive_diff.added(), ["a.g", "h", "i"])

    def test_removed(self):
        self.assertEqual(self.recursive_diff.removed(), ["a.f"])

    def test_changed_with_ignore_unset_values(self):
        self.recursive_diff.ignore_unset_values = True
        self.assertEqual(self.recursive_diff.changed(), ["a.c", "a.e"])

    def test_changed_without_ignore_unset_values(self):
        self.recursive_diff.ignore_unset_values = False
        self.assertEqual(
            self.recursive_diff.changed(), ["a.c", "a.e", "a.f", "a.g", "h", "i"]
        )

    def test_unchanged(self):
        self.assertEqual(self.recursive_diff.unchanged(), ["a.b", "j"])

    def test_diffs(self):
        self.assertDictEqual(
            self.recursive_diff.diffs,
            {
                "a": {
                    "c": {"old": 2, "new": 4},
                    "e": {"old": "old_value", "new": "new_value"},
                    "f": {"old": "old_key", "new": NONE},
                    "g": {"old": NONE, "new": "new_key"},
                },
                "h": {"old": NONE, "new": "new_key"},
                "i": {"old": NONE, "new": None},
            },
        )
        self.assertDictEqual(
            self.recursive_diff_ign.diffs,
            {
                "a": {
                    "c": {"old": 2, "new": 4},
                    "e": {"old": "old_value", "new": "new_value"},
                    "g": {"old": NONE, "new": "new_key"},
                },
                "h": {"old": NONE, "new": "new_key"},
                "i": {"old": NONE, "new": None},
            },
        )

    def test_new_values(self):
        self.assertDictEqual(
            self.recursive_diff.new_values,
            {
                "a": {"c": 4, "e": "new_value", "f": NONE, "g": "new_key"},
                "h": "new_key",
                "i": None,
            },
        )

    def test_old_values(self):
        self.assertDictEqual(
            self.recursive_diff.old_values,
            {
                "a": {"c": 2, "e": "old_value", "f": "old_key", "g": NONE},
                "h": NONE,
                "i": NONE,
            },
        )

    def test_changes_str(self):
        self.assertEqual(
            self.recursive_diff.changes_str,
            "a:\n"
            "  c from 2 to 4\n"
            "  e from 'old_value' to 'new_value'\n"
            "  f from 'old_key' to nothing\n"
            "  g from nothing to 'new_key'\n"
            "h from nothing to 'new_key'\n"
            "i from nothing to None",
        )



# -*- coding: utf-8 -*-

from __future__ import absolute_import

import pytest
import salt.utils.path
import salt.utils.platform
from tests.support.case import ModuleCase
from tests.support.helpers import slowTest
from tests.support.unit import skipIf

URL = "google-public-dns-a.google.com"


@pytest.mark.windows_whitelisted
class NetworkTest(ModuleCase):
    """
    Validate network module
    """

    @slowTest
    def test_network_ping(self):
        """
        network.ping
        """
        ret = self.run_function("network.ping", [URL])
        exp_out = ["ping", URL, "ms", "time"]
        for out in exp_out:
            self.assertIn(out, ret.lower())

    @skipIf(salt.utils.platform.is_darwin(), "not supported on macosx")
    @slowTest
    def test_network_netstat(self):
        """
        network.netstat
        """
        ret = self.run_function("network.netstat")
        exp_out = ["proto", "local-address"]
        for val in ret:
            for out in exp_out:
                self.assertIn(out, val)

    @slowTest
    def test_network_traceroute(self):
        """
        network.traceroute
        """
        if (
            not salt.utils.path.which("traceroute")
            and not salt.utils.platform.is_windows()
        ):
            self.skipTest("traceroute not installed")
        ret = self.run_function("network.traceroute", [URL])
        exp_out = ["hostname", "ip"]
        for out in exp_out:
            self.assertIn(out, exp_out)

    @skipIf(not salt.utils.platform.is_windows(), "windows only test")
    @slowTest
    def test_network_nslookup(self):
        """
        network.nslookup
        """
        ret = self.run_function("network.nslookup", [URL])
        exp_out = ["Server", "Address"]
        for out in exp_out:
            self.assertIn(out, exp_out)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import os
import threading
import time

import pytest
import salt.utils.json
import salt.utils.stringutils
from salt.ext import six
from salt.netapi.rest_tornado import saltnado
from salt.utils.versions import StrictVersion
from salt.utils.zeromq import ZMQDefaultLoop as ZMQIOLoop
from salt.utils.zeromq import zmq
from tests.support.helpers import flaky, slowTest
from tests.support.unit import skipIf
from tests.unit.netapi.test_rest_tornado import SaltnadoTestCase

HAS_ZMQ_IOLOOP = bool(zmq)


class _SaltnadoIntegrationTestCase(SaltnadoTestCase):  # pylint: disable=abstract-method
    @property
    def opts(self):
        return self.get_config("client_config", from_scratch=True)

    @property
    def mod_opts(self):
        return self.get_config("minion", from_scratch=True)


@skipIf(HAS_ZMQ_IOLOOP is False, "PyZMQ version must be >= 14.0.1 to run these tests.")
@skipIf(
    StrictVersion(zmq.__version__) < StrictVersion("14.0.1"),
    "PyZMQ must be >= 14.0.1 to run these tests.",
)
@pytest.mark.usefixtures("salt_sub_minion")
class TestSaltAPIHandler(_SaltnadoIntegrationTestCase):
    def setUp(self):
        super(TestSaltAPIHandler, self).setUp()
        os.environ["ASYNC_TEST_TIMEOUT"] = "300"

    def get_app(self):
        urls = [("/", saltnado.SaltAPIHandler)]

        application = self.build_tornado_app(urls)

        application.event_listener = saltnado.EventListener({}, self.opts)
        self.application = application
        return application

    def test_root(self):
        """
        Test the root path which returns the list of clients we support
        """
        response = self.fetch("/", connect_timeout=30, request_timeout=30,)
        self.assertEqual(response.code, 200)
        response_obj = salt.utils.json.loads(response.body)
        self.assertEqual(
            sorted(response_obj["clients"]),
            ["local", "local_async", "runner", "runner_async"],
        )
        self.assertEqual(response_obj["return"], "Welcome")

    @slowTest
    def test_post_no_auth(self):
        """
        Test post with no auth token, should 401
        """
        # get a token for this test
        low = [{"client": "local", "tgt": "*", "fun": "test.ping"}]
        response = self.fetch(
            "/",
            method="POST",
            body=salt.utils.json.dumps(low),
            headers={"Content-Type": self.content_type_map["json"]},
            follow_redirects=False,
            connect_timeout=30,
            request_timeout=30,
        )
        self.assertEqual(response.code, 302)
        self.assertEqual(response.headers["Location"], "/login")

    # Local client tests

    @skipIf(True, "to be re-enabled when #23623 is merged")
    def test_simple_local_post(self):
        """
        Test a basic API of /
        """
        low = [{"client": "local", "tgt": "*", "fun": "test.ping"}]
        response = self.fetch(
            "/",
            method="POST",
            body=salt.utils.json.dumps(low),
            headers={
                "Content-Type": self.content_type_map["json"],
                saltnado.AUTH_TOKEN_HEADER: self.token["token"],
            },
            connect_timeout=30,
            request_timeout=30,
        )
        response_obj = salt.utils.json.loads(response.body)
        self.assertEqual(len(response_obj["return"]), 1)
        # If --proxy is set, it will cause an extra minion_id to be in the
        # response. Since there's not a great way to know if the test
        # runner's proxy minion is running, and we're not testing proxy
        # minions here anyway, just remove it from the response.
        response_obj["return"][0].pop("proxytest", None)
        self.assertEqual(
            response_obj["return"][0], {"minion": True, "sub_minion": True}
        )

    def test_simple_local_post_no_tgt(self):
        """
        POST job with invalid tgt
        """
        low = [{"client": "local", "tgt": "minion_we_dont_have", "fun": "test.ping"}]
        response = self.fetch(
            "/",
            method="POST",
            body=salt.utils.json.dumps(low),
            headers={
                "Content-Type": self.content_type_map["json"],
                saltnado.AUTH_TOKEN_HEADER: self.token["token"],
            },
            connect_timeout=30,
            request_timeout=30,
        )
        response_obj = salt.utils.json.loads(response.body)
        self.assertEqual(
            response_obj["return"],
            [
                "No minions matched the target. No command was sent, no jid was assigned."
            ],
        )

    # local client request body test

    @skipIf(True, "Undetermined race condition in test. Temporarily disabled.")
    def test_simple_local_post_only_dictionary_request(self):
        """
        Test a basic API of /
        """
        low = {
            "client": "local",
            "tgt": "*",
            "fun": "test.ping",
        }
        response = self.fetch(
            "/",
            method="POST",
            body=salt.utils.json.dumps(low),
            headers={
                "Content-Type": self.content_type_map["json"],
                saltnado.AUTH_TOKEN_HEADER: self.token["token"],
            },
            connect_timeout=30,
            request_timeout=30,
        )
        response_obj = salt.utils.json.loads(response.body)
        self.assertEqual(len(response_obj["return"]), 1)
        # If --proxy is set, it will cause an extra minion_id to be in the
        # response. Since there's not a great way to know if the test
        # runner's proxy minion is running, and we're not testing proxy
        # minions here anyway, just remove it from the response.
        response_obj["return"][0].pop("proxytest", None)
        self.assertEqual(
            response_obj["return"][0], {"minion": True, "sub_minion": True}
        )

    def test_simple_local_post_invalid_request(self):
        """
        Test a basic API of /
        """
        low = ["invalid request"]
        response = self.fetch(
            "/",
            method="POST",
            body=salt.utils.json.dumps(low),
            headers={
                "Content-Type": self.content_type_map["json"],
                saltnado.AUTH_TOKEN_HEADER: self.token["token"],
            },
            connect_timeout=30,
            request_timeout=30,
        )
        self.assertEqual(response.code, 400)

    # local_async tests
    def test_simple_local_async_post(self):
        low = [{"client": "local_async", "tgt": "*", "fun": "test.ping"}]
        response = self.fetch(
            "/",
            method="POST",
            body=salt.utils.json.dumps(low),
            headers={
                "Content-Type": self.content_type_map["json"],
                saltnado.AUTH_TOKEN_HEADER: self.token["token"],
            },
        )

        response_obj = salt.utils.json.loads(response.body)
        ret = response_obj["return"]
        ret[0]["minions"] = sorted(ret[0]["minions"])
        try:
            # If --proxy is set, it will cause an extra minion_id to be in the
            # response. Since there's not a great way to know if the test
            # runner's proxy minion is running, and we're not testing proxy
            # minions here anyway, just remove it from the response.
            ret[0]["minions"].remove("proxytest")
        except ValueError:
            pass

        # TODO: verify pub function? Maybe look at how we test the publisher
        self.assertEqual(len(ret), 1)
        self.assertIn("jid", ret[0])
        self.assertEqual(ret[0]["minions"], sorted(["minion", "sub_minion"]))

    def test_multi_local_async_post(self):
        low = [
            {"client": "local_async", "tgt": "*", "fun": "test.ping"},
            {"client": "local_async", "tgt": "*", "fun": "test.ping"},
        ]
        response = self.fetch(
            "/",
            method="POST",
            body=salt.utils.json.dumps(low),
            headers={
                "Content-Type": self.content_type_map["json"],
                saltnado.AUTH_TOKEN_HEADER: self.token["token"],
            },
        )

        response_obj = salt.utils.json.loads(response.body)
        ret = response_obj["return"]
        ret[0]["minions"] = sorted(ret[0]["minions"])
        ret[1]["minions"] = sorted(ret[1]["minions"])
        try:
            # If --proxy is set, it will cause an extra minion_id to be in the
            # response. Since there's not a great way to know if the test
            # runner's proxy minion is running, and we're not testing proxy
            # minions here anyway, just remove it from the response.
            ret[0]["minions"].remove("proxytest")
            ret[1]["minions"].remove("proxytest")
        except ValueError:
            pass

        self.assertEqual(len(ret), 2)
        self.assertIn("jid", ret[0])
        self.assertIn("jid", ret[1])
        self.assertEqual(ret[0]["minions"], sorted(["minion", "sub_minion"]))
        self.assertEqual(ret[1]["minions"], sorted(["minion", "sub_minion"]))

    @slowTest
    def test_multi_local_async_post_multitoken(self):
        low = [
            {"client": "local_async", "tgt": "*", "fun": "test.ping"},
            {
                "client": "local_async",
                "tgt": "*",
                "fun": "test.ping",
                "token": self.token[
                    "token"
                ],  # send a different (but still valid token)
            },
            {
                "client": "local_async",
                "tgt": "*",
                "fun": "test.ping",
                "token": "BAD_TOKEN",  # send a bad token
            },
        ]
        response = self.fetch(
            "/",
            method="POST",
            body=salt.utils.json.dumps(low),
            headers={
                "Content-Type": self.content_type_map["json"],
                saltnado.AUTH_TOKEN_HEADER: self.token["token"],
            },
        )

        response_obj = salt.utils.json.loads(response.body)
        ret = response_obj["return"]
        ret[0]["minions"] = sorted(ret[0]["minions"])
        ret[1]["minions"] = sorted(ret[1]["minions"])
        try:
            # If --proxy is set, it will cause an extra minion_id to be in the
            # response. Since there's not a great way to know if the test
            # runner's proxy minion is running, and we're not testing proxy
            # minions here anyway, just remove it from the response.
            ret[0]["minions"].remove("proxytest")
            ret[1]["minions"].remove("proxytest")
        except ValueError:
            pass

        self.assertEqual(len(ret), 3)  # make sure we got 3 responses
        self.assertIn("jid", ret[0])  # the first 2 are regular returns
        self.assertIn("jid", ret[1])
        self.assertIn("Failed to authenticate", ret[2])  # bad auth
        self.assertEqual(ret[0]["minions"], sorted(["minion", "sub_minion"]))
        self.assertEqual(ret[1]["minions"], sorted(["minion", "sub_minion"]))

    @slowTest
    def test_simple_local_async_post_no_tgt(self):
        low = [
            {"client": "local_async", "tgt": "minion_we_dont_have", "fun": "test.ping"}
        ]
        response = self.fetch(
            "/",
            method="POST",
            body=salt.utils.json.dumps(low),
            headers={
                "Content-Type": self.content_type_map["json"],
                saltnado.AUTH_TOKEN_HEADER: self.token["token"],
            },
        )
        response_obj = salt.utils.json.loads(response.body)
        self.assertEqual(response_obj["return"], [{}])

    @skipIf(True, "Undetermined race condition in test. Temporarily disabled.")
    def test_simple_local_post_only_dictionary_request_with_order_masters(self):
        """
        Test a basic API of /
        """
        low = {
            "client": "local",
            "tgt": "*",
            "fun": "test.ping",
        }

        self.application.opts["order_masters"] = True
        self.application.opts["syndic_wait"] = 5

        response = self.fetch(
            "/",
            method="POST",
            body=salt.utils.json.dumps(low),
            headers={
                "Content-Type": self.content_type_map["json"],
                saltnado.AUTH_TOKEN_HEADER: self.token["token"],
            },
            connect_timeout=30,
            request_timeout=30,
        )
        response_obj = salt.utils.json.loads(response.body)
        self.application.opts["order_masters"] = []
        self.application.opts["syndic_wait"] = 5
        # If --proxy is set, it will cause an extra minion_id to be in the
        # response. Since there's not a great way to know if the test runner's
        # proxy minion is running, and we're not testing proxy minions here
        # anyway, just remove it from the response.
        response_obj[0]["return"].pop("proxytest", None)
        self.assertEqual(response_obj["return"], [{"minion": True, "sub_minion": True}])

    # runner tests
    @slowTest
    def test_simple_local_runner_post(self):
        low = [{"client": "runner", "fun": "manage.up"}]
        response = self.fetch(
            "/",
            method="POST",
            body=salt.utils.json.dumps(low),
            headers={
                "Content-Type": self.content_type_map["json"],
                saltnado.AUTH_TOKEN_HEADER: self.token["token"],
            },
            connect_timeout=30,
            request_timeout=300,
        )
        response_obj = salt.utils.json.loads(response.body)
        self.assertEqual(len(response_obj["return"]), 1)
        try:
            # If --proxy is set, it will cause an extra minion_id to be in the
            # response. Since there's not a great way to know if the test
            # runner's proxy minion is running, and we're not testing proxy
            # minions here anyway, just remove it from the response.
            response_obj["return"][0].remove("proxytest")
        except ValueError:
            pass
        self.assertEqual(
            sorted(response_obj["return"][0]), sorted(["minion", "sub_minion"])
        )

    # runner_async tests
    def test_simple_local_runner_async_post(self):
        low = [{"client": "runner_async", "fun": "manage.up"}]
        response = self.fetch(
            "/",
            method="POST",
            body=salt.utils.json.dumps(low),
            headers={
                "Content-Type": self.content_type_map["json"],
                saltnado.AUTH_TOKEN_HEADER: self.token["token"],
            },
            connect_timeout=10,
            request_timeout=10,
        )
        response_obj = salt.utils.json.loads(response.body)
        self.assertIn("return", response_obj)
        self.assertEqual(1, len(response_obj["return"]))
        self.assertIn("jid", response_obj["return"][0])
        self.assertIn("tag", response_obj["return"][0])


@flaky
@skipIf(HAS_ZMQ_IOLOOP is False, "PyZMQ version must be >= 14.0.1 to run these tests.")
class TestMinionSaltAPIHandler(_SaltnadoIntegrationTestCase):
    def get_app(self):
        urls = [
            (r"/minions/(.*)", saltnado.MinionSaltAPIHandler),
            (r"/minions", saltnado.MinionSaltAPIHandler),
        ]
        application = self.build_tornado_app(urls)
        application.event_listener = saltnado.EventListener({}, self.opts)
        return application

    @skipIf(True, "issue #34753")
    def test_get_no_mid(self):
        response = self.fetch(
            "/minions",
            method="GET",
            headers={saltnado.AUTH_TOKEN_HEADER: self.token["token"]},
            follow_redirects=False,
        )
        response_obj = salt.utils.json.loads(response.body)
        self.assertEqual(len(response_obj["return"]), 1)
        # one per minion
        self.assertEqual(len(response_obj["return"][0]), 2)
        # check a single grain
        for minion_id, grains in six.iteritems(response_obj["return"][0]):
            self.assertEqual(minion_id, grains["id"])

    @skipIf(True, "to be re-enabled when #23623 is merged")
    @slowTest
    def test_get(self):
        response = self.fetch(
            "/minions/minion",
            method="GET",
            headers={saltnado.AUTH_TOKEN_HEADER: self.token["token"]},
            follow_redirects=False,
        )
        response_obj = salt.utils.json.loads(response.body)
        self.assertEqual(len(response_obj["return"]), 1)
        self.assertEqual(len(response_obj["return"][0]), 1)
        # check a single grain
        self.assertEqual(response_obj["return"][0]["minion"]["id"], "minion")

    def test_post(self):
        low = [{"tgt": "*minion", "fun": "test.ping"}]
        response = self.fetch(
            "/minions",
            method="POST",
            body=salt.utils.json.dumps(low),
            headers={
                "Content-Type": self.content_type_map["json"],
                saltnado.AUTH_TOKEN_HEADER: self.token["token"],
            },
        )

        response_obj = salt.utils.json.loads(response.body)
        ret = response_obj["return"]
        ret[0]["minions"] = sorted(ret[0]["minions"])

        # TODO: verify pub function? Maybe look at how we test the publisher
        self.assertEqual(len(ret), 1)
        self.assertIn("jid", ret[0])
        self.assertEqual(ret[0]["minions"], sorted(["minion", "sub_minion"]))

    @slowTest
    def test_post_with_client(self):
        # get a token for this test
        low = [{"client": "local_async", "tgt": "*minion", "fun": "test.ping"}]
        response = self.fetch(
            "/minions",
            method="POST",
            body=salt.utils.json.dumps(low),
            headers={
                "Content-Type": self.content_type_map["json"],
                saltnado.AUTH_TOKEN_HEADER: self.token["token"],
            },
        )

        response_obj = salt.utils.json.loads(response.body)
        ret = response_obj["return"]
        ret[0]["minions"] = sorted(ret[0]["minions"])

        # TODO: verify pub function? Maybe look at how we test the publisher
        self.assertEqual(len(ret), 1)
        self.assertIn("jid", ret[0])
        self.assertEqual(ret[0]["minions"], sorted(["minion", "sub_minion"]))

    @slowTest
    def test_post_with_incorrect_client(self):
        """
        The /minions endpoint is asynchronous only, so if you try something else
        make sure you get an error
        """
        # get a token for this test
        low = [{"client": "local", "tgt": "*", "fun": "test.ping"}]
        response = self.fetch(
            "/minions",
            method="POST",
            body=salt.utils.json.dumps(low),
            headers={
                "Content-Type": self.content_type_map["json"],
                saltnado.AUTH_TOKEN_HEADER: self.token["token"],
            },
        )
        self.assertEqual(response.code, 400)


@skipIf(HAS_ZMQ_IOLOOP is False, "PyZMQ version must be >= 14.0.1 to run these tests.")
class TestJobsSaltAPIHandler(_SaltnadoIntegrationTestCase):
    def get_app(self):
        urls = [
            (r"/jobs/(.*)", saltnado.JobsSaltAPIHandler),
            (r"/jobs", saltnado.JobsSaltAPIHandler),
        ]
        application = self.build_tornado_app(urls)
        application.event_listener = saltnado.EventListener({}, self.opts)
        return application

    @skipIf(True, "to be re-enabled when #23623 is merged")
    @slowTest
    def test_get(self):
        # test with no JID
        self.http_client.fetch(
            self.get_url("/jobs"),
            self.stop,
            method="GET",
            headers={saltnado.AUTH_TOKEN_HEADER: self.token["token"]},
            follow_redirects=False,
        )
        response = self.wait(timeout=30)
        response_obj = salt.utils.json.loads(response.body)["return"][0]
        try:
            for jid, ret in six.iteritems(response_obj):
                self.assertIn("Function", ret)
                self.assertIn("Target", ret)
                self.assertIn("Target-type", ret)
                self.assertIn("User", ret)
                self.assertIn("StartTime", ret)
                self.assertIn("Arguments", ret)
        except AttributeError as attribute_error:
            print(salt.utils.json.loads(response.body))
            raise

        # test with a specific JID passed in
        jid = next(six.iterkeys(response_obj))
        self.http_client.fetch(
            self.get_url("/jobs/{0}".format(jid)),
            self.stop,
            method="GET",
            headers={saltnado.AUTH_TOKEN_HEADER: self.token["token"]},
            follow_redirects=False,
        )
        response = self.wait(timeout=30)
        response_obj = salt.utils.json.loads(response.body)["return"][0]
        self.assertIn("Function", response_obj)
        self.assertIn("Target", response_obj)
        self.assertIn("Target-type", response_obj)
        self.assertIn("User", response_obj)
        self.assertIn("StartTime", response_obj)
        self.assertIn("Arguments", response_obj)
        self.assertIn("Result", response_obj)


# TODO: run all the same tests from the root handler, but for now since they are
# the same code, we'll just sanity check
@skipIf(HAS_ZMQ_IOLOOP is False, "PyZMQ version must be >= 14.0.1 to run these tests.")
class TestRunSaltAPIHandler(_SaltnadoIntegrationTestCase):
    def get_app(self):
        urls = [
            ("/run", saltnado.RunSaltAPIHandler),
        ]
        application = self.build_tornado_app(urls)
        application.event_listener = saltnado.EventListener({}, self.opts)
        return application

    @skipIf(True, "to be re-enabled when #23623 is merged")
    @slowTest
    def test_get(self):
        low = [{"client": "local", "tgt": "*", "fun": "test.ping"}]
        response = self.fetch(
            "/run",
            method="POST",
            body=salt.utils.json.dumps(low),
            headers={
                "Content-Type": self.content_type_map["json"],
                saltnado.AUTH_TOKEN_HEADER: self.token["token"],
            },
        )
        response_obj = salt.utils.json.loads(response.body)
        self.assertEqual(response_obj["return"], [{"minion": True, "sub_minion": True}])


@skipIf(HAS_ZMQ_IOLOOP is False, "PyZMQ version must be >= 14.0.1 to run these tests.")
class TestEventsSaltAPIHandler(_SaltnadoIntegrationTestCase):
    def get_app(self):
        urls = [
            (r"/events", saltnado.EventsSaltAPIHandler),
        ]
        application = self.build_tornado_app(urls)
        application.event_listener = saltnado.EventListener({}, self.opts)

        # store a reference, for magic later!
        self.application = application
        self.events_to_fire = 0
        return application

    @slowTest
    def test_get(self):
        self.events_to_fire = 5
        response = self.fetch(
            "/events",
            headers={saltnado.AUTH_TOKEN_HEADER: self.token["token"]},
            streaming_callback=self.on_event,
        )

    def _stop(self):
        self.stop()

    def on_event(self, event):
        if six.PY3:
            event = event.decode("utf-8")
        if self.events_to_fire > 0:
            self.application.event_listener.event.fire_event(
                {"foo": "bar", "baz": "qux"}, "salt/netapi/test"
            )
            self.events_to_fire -= 1
        # once we've fired all the events, lets call it a day
        else:
            # wait so that we can ensure that the next future is ready to go
            # to make sure we don't explode if the next one is ready
            ZMQIOLoop.current().add_timeout(time.time() + 0.5, self._stop)

        event = event.strip()
        # if we got a retry, just continue
        if event != "retry: 400":
            tag, data = event.splitlines()
            self.assertTrue(tag.startswith("tag: "))
            self.assertTrue(data.startswith("data: "))


@skipIf(HAS_ZMQ_IOLOOP is False, "PyZMQ version must be >= 14.0.1 to run these tests.")
class TestWebhookSaltAPIHandler(_SaltnadoIntegrationTestCase):
    def get_app(self):

        urls = [
            (r"/hook(/.*)?", saltnado.WebhookSaltAPIHandler),
        ]

        application = self.build_tornado_app(urls)

        self.application = application

        application.event_listener = saltnado.EventListener({}, self.opts)
        return application

    @skipIf(True, "Skipping until we can devote more resources to debugging this test.")
    def test_post(self):
        self._future_resolved = threading.Event()
        try:

            def verify_event(future):
                """
                Notify the threading event that the future is resolved
                """
                self._future_resolved.set()

            self._finished = (
                False  # TODO: remove after some cleanup of the event listener
            )

            # get an event future
            future = self.application.event_listener.get_event(
                self, tag="salt/netapi/hook", callback=verify_event
            )
            # fire the event
            response = self.fetch(
                "/hook",
                method="POST",
                body="foo=bar",
                headers={saltnado.AUTH_TOKEN_HEADER: self.token["token"]},
            )
            response_obj = salt.utils.json.loads(response.body)
            self.assertTrue(response_obj["success"])
            resolve_future_timeout = 60
            self._future_resolved.wait(resolve_future_timeout)
            try:
                event = future.result()
            except Exception as exc:  # pylint: disable=broad-except
                self.fail(
                    "Failed to resolve future under {} secs: {}".format(
                        resolve_future_timeout, exc
                    )
                )
            self.assertEqual(event["tag"], "salt/netapi/hook")
            self.assertIn("headers", event["data"])
            self.assertEqual(
                event["data"]["post"], {"foo": salt.utils.stringutils.to_bytes("bar")}
            )
        finally:
            self._future_resolved.clear()
            del self._future_resolved



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jochen Breuer <jbreuer@suse.de>
"""
# pylint: disable=no-value-for-parameter

# Import Python Libs
from __future__ import absolute_import

import os
from contextlib import contextmanager

import salt.utils.files
import salt.utils.platform
from salt.modules import kubernetesmod as kubernetes

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import Mock, patch
from tests.support.unit import TestCase, skipIf


@contextmanager
def mock_kubernetes_library():
    """
    After fixing the bug in 1c821c0e77de58892c77d8e55386fac25e518c31,
    it caused kubernetes._cleanup() to get called for virtually every
    test, which blows up. This prevents that specific blow-up once
    """
    with patch("salt.modules.kubernetesmod.kubernetes") as mock_kubernetes_lib:
        yield mock_kubernetes_lib


@skipIf(
    not kubernetes.HAS_LIBS,
    "Kubernetes client lib is not installed. " "Skipping test_kubernetes.py",
)
class KubernetesTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.kubernetesmod
    """

    def setup_loader_modules(self):
        return {kubernetes: {"__salt__": {}}}

    def test_nodes(self):
        """
        Test node listing.
        :return:
        """
        with mock_kubernetes_library() as mock_kubernetes_lib:
            with patch.dict(
                kubernetes.__salt__, {"config.option": Mock(side_effect=self.settings)}
            ):
                mock_kubernetes_lib.client.CoreV1Api.return_value = Mock(
                    **{
                        "list_node.return_value.to_dict.return_value": {
                            "items": [{"metadata": {"name": "mock_node_name"}}]
                        }
                    }
                )
                self.assertEqual(kubernetes.nodes(), ["mock_node_name"])
                self.assertTrue(
                    kubernetes.kubernetes.client.CoreV1Api().list_node().to_dict.called
                )

    def test_deployments(self):
        """
        Tests deployment listing.
        :return:
        """
        with mock_kubernetes_library() as mock_kubernetes_lib:
            with patch.dict(
                kubernetes.__salt__, {"config.option": Mock(side_effect=self.settings)}
            ):
                mock_kubernetes_lib.client.ExtensionsV1beta1Api.return_value = Mock(
                    **{
                        "list_namespaced_deployment.return_value.to_dict.return_value": {
                            "items": [{"metadata": {"name": "mock_deployment_name"}}]
                        }
                    }
                )
                self.assertEqual(kubernetes.deployments(), ["mock_deployment_name"])
                # pylint: disable=E1120
                self.assertTrue(
                    kubernetes.kubernetes.client.ExtensionsV1beta1Api()
                    .list_namespaced_deployment()
                    .to_dict.called
                )
                # pylint: enable=E1120

    def test_services(self):
        """
        Tests services listing.
        :return:
        """
        with mock_kubernetes_library() as mock_kubernetes_lib:
            with patch.dict(
                kubernetes.__salt__, {"config.option": Mock(side_effect=self.settings)}
            ):
                mock_kubernetes_lib.client.CoreV1Api.return_value = Mock(
                    **{
                        "list_namespaced_service.return_value.to_dict.return_value": {
                            "items": [{"metadata": {"name": "mock_service_name"}}]
                        }
                    }
                )
                self.assertEqual(kubernetes.services(), ["mock_service_name"])
                # pylint: disable=E1120
                self.assertTrue(
                    kubernetes.kubernetes.client.CoreV1Api()
                    .list_namespaced_service()
                    .to_dict.called
                )
                # pylint: enable=E1120

    def test_pods(self):
        """
        Tests pods listing.
        :return:
        """
        with mock_kubernetes_library() as mock_kubernetes_lib:
            with patch.dict(
                kubernetes.__salt__, {"config.option": Mock(side_effect=self.settings)}
            ):
                mock_kubernetes_lib.client.CoreV1Api.return_value = Mock(
                    **{
                        "list_namespaced_pod.return_value.to_dict.return_value": {
                            "items": [{"metadata": {"name": "mock_pod_name"}}]
                        }
                    }
                )
                self.assertEqual(kubernetes.pods(), ["mock_pod_name"])
                # pylint: disable=E1120
                self.assertTrue(
                    kubernetes.kubernetes.client.CoreV1Api()
                    .list_namespaced_pod()
                    .to_dict.called
                )
                # pylint: enable=E1120

    def test_delete_deployments(self):
        """
        Tests deployment deletion
        :return:
        """
        with mock_kubernetes_library() as mock_kubernetes_lib:
            with patch(
                "salt.modules.kubernetesmod.show_deployment", Mock(return_value=None)
            ):
                with patch.dict(
                    kubernetes.__salt__,
                    {"config.option": Mock(side_effect=self.settings)},
                ):
                    mock_kubernetes_lib.client.V1DeleteOptions = Mock(return_value="")
                    mock_kubernetes_lib.client.ExtensionsV1beta1Api.return_value = Mock(
                        **{
                            "delete_namespaced_deployment.return_value.to_dict.return_value": {
                                "code": ""
                            }
                        }
                    )
                    self.assertEqual(
                        kubernetes.delete_deployment("test"), {"code": 200}
                    )
                    # pylint: disable=E1120
                    self.assertTrue(
                        kubernetes.kubernetes.client.ExtensionsV1beta1Api()
                        .delete_namespaced_deployment()
                        .to_dict.called
                    )
                    # pylint: enable=E1120

    def test_create_deployments(self):
        """
        Tests deployment creation.
        :return:
        """
        with mock_kubernetes_library() as mock_kubernetes_lib:
            with patch.dict(
                kubernetes.__salt__, {"config.option": Mock(side_effect=self.settings)}
            ):
                mock_kubernetes_lib.client.ExtensionsV1beta1Api.return_value = Mock(
                    **{
                        "create_namespaced_deployment.return_value.to_dict.return_value": {}
                    }
                )
                self.assertEqual(
                    kubernetes.create_deployment(
                        "test", "default", {}, {}, None, None, None
                    ),
                    {},
                )
                # pylint: disable=E1120
                self.assertTrue(
                    kubernetes.kubernetes.client.ExtensionsV1beta1Api()
                    .create_namespaced_deployment()
                    .to_dict.called
                )
                # pylint: enable=E1120

    @staticmethod
    def settings(name, value=None):
        """
        Test helper
        :return: settings or default
        """
        data = {
            "kubernetes.kubeconfig": "/home/testuser/.minikube/kubeconfig.cfg",
            "kubernetes.context": "minikube",
        }
        return data.get(name, value)

    def test_setup_kubeconfig_file(self):
        """
        Test that the `kubernetes.kubeconfig` configuration isn't overwritten
        :return:
        """
        with mock_kubernetes_library() as mock_kubernetes_lib:
            with patch.dict(
                kubernetes.__salt__, {"config.option": Mock(side_effect=self.settings)}
            ):
                mock_kubernetes_lib.config.load_kube_config = Mock()
                config = kubernetes._setup_conn()
                self.assertEqual(
                    self.settings("kubernetes.kubeconfig"), config["kubeconfig"],
                )

    def test_setup_kubeconfig_data_overwrite(self):
        """
        Test that provided `kubernetes.kubeconfig` configuration is overwritten
        by provided kubeconfig_data in the command
        :return:
        """
        with mock_kubernetes_library() as mock_kubernetes_lib:
            with patch.dict(
                kubernetes.__salt__, {"config.option": Mock(side_effect=self.settings)}
            ):
                mock_kubernetes_lib.config.load_kube_config = Mock()
                config = kubernetes._setup_conn(
                    kubeconfig_data="MTIzNDU2Nzg5MAo=", context="newcontext"
                )
                check_path = os.path.join("/tmp", "salt-kubeconfig-")
                if salt.utils.platform.is_windows():
                    check_path = os.path.join(os.environ.get("TMP"), "salt-kubeconfig-")
                elif salt.utils.platform.is_darwin():
                    check_path = os.path.join(
                        os.environ.get("TMPDIR", "/tmp"), "salt-kubeconfig-"
                    )
                self.assertTrue(
                    config["kubeconfig"].lower().startswith(check_path.lower())
                )
                self.assertTrue(os.path.exists(config["kubeconfig"]))
                with salt.utils.files.fopen(config["kubeconfig"], "r") as kcfg:
                    self.assertEqual("1234567890\n", kcfg.read())
                kubernetes._cleanup(**config)

    def test_node_labels(self):
        """
        Test kubernetes.node_labels
        :return:
        """
        with patch("salt.modules.kubernetesmod.node") as mock_node:
            mock_node.return_value = {
                "metadata": {
                    "labels": {
                        "kubernetes.io/hostname": "minikube",
                        "kubernetes.io/os": "linux",
                    }
                }
            }
            self.assertEqual(
                kubernetes.node_labels("minikube"),
                {"kubernetes.io/hostname": "minikube", "kubernetes.io/os": "linux"},
            )

    def test_adding_change_cause_annotation(self):
        """
        Tests adding a `kubernetes.io/change-cause` annotation just like
        kubectl [apply|create|replace] --record does
        :return:
        """
        with patch(
            "salt.modules.kubernetesmod.sys.argv", ["/usr/bin/salt-call", "state.apply"]
        ) as mock_sys:
            func = getattr(kubernetes, "__dict_to_object_meta")
            data = func(name="test-pod", namespace="test", metadata={})

            self.assertEqual(data.name, "test-pod")
            self.assertEqual(data.namespace, "test")
            self.assertEqual(
                data.annotations,
                {"kubernetes.io/change-cause": "/usr/bin/salt-call state.apply"},
            )

            # Ensure any specified annotations aren't overwritten
            test_metadata = {"annotations": {"kubernetes.io/change-cause": "NOPE"}}
            data = func(name="test-pod", namespace="test", metadata=test_metadata)

            self.assertEqual(data.annotations, {"kubernetes.io/change-cause": "NOPE"})

    def test_enforce_only_strings_dict(self):
        func = getattr(kubernetes, "__enforce_only_strings_dict")
        data = {
            "unicode": 1,
            2: 2,
        }
        self.assertEqual(
            {"unicode": "1", "2": "2"}, func(data),
        )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.boto_dynamodb as boto_dynamodb

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class BotoDynamodbTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.boto_dynamodb
    """

    def setup_loader_modules(self):
        return {boto_dynamodb: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure the DynamoDB table exists.
        """
        name = "new_table"

        ret = {"name": name, "result": True, "changes": {}, "comment": ""}

        exists_mock = MagicMock(side_effect=[True, False, False])
        dict_mock = MagicMock(return_value={})
        mock_bool = MagicMock(return_value=True)
        pillar_mock = MagicMock(return_value=[])
        with patch.dict(
            boto_dynamodb.__salt__,
            {
                "boto_dynamodb.exists": exists_mock,
                "boto_dynamodb.describe": dict_mock,
                "config.option": dict_mock,
                "pillar.get": pillar_mock,
                "boto_dynamodb.create_table": mock_bool,
            },
        ):
            comt = (
                "DynamoDB table {0} exists,\n"
                "DynamoDB table {0} throughput matches,\n"
                "All global secondary indexes match,\n".format(name)
            )
            ret.update({"comment": comt})
            self.assertDictEqual(boto_dynamodb.present(name), ret)

            with patch.dict(boto_dynamodb.__opts__, {"test": True}):
                comt = "DynamoDB table {0} would be created.".format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(boto_dynamodb.present(name), ret)

            changes = {
                "new": {
                    "global_indexes": None,
                    "hash_key": None,
                    "hash_key_data_type": None,
                    "local_indexes": None,
                    "range_key": None,
                    "range_key_data_type": None,
                    "read_capacity_units": None,
                    "table": "new_table",
                    "write_capacity_units": None,
                }
            }

            with patch.dict(boto_dynamodb.__opts__, {"test": False}):
                comt = (
                    "DynamoDB table {0} was successfully created,\n"
                    "DynamoDB table new_table throughput matches,\n".format(name)
                )
                ret.update({"comment": comt, "result": True, "changes": changes})
                self.assertDictEqual(ret, boto_dynamodb.present(name))

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure the DynamoDB table does not exist.
        """
        name = "new_table"

        ret = {"name": name, "result": True, "changes": {}, "comment": ""}

        mock = MagicMock(side_effect=[False, True, True])
        mock_bool = MagicMock(return_value=True)
        with patch.dict(
            boto_dynamodb.__salt__,
            {"boto_dynamodb.exists": mock, "boto_dynamodb.delete": mock_bool},
        ):
            comt = "DynamoDB table {0} does not exist".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(boto_dynamodb.absent(name), ret)

            with patch.dict(boto_dynamodb.__opts__, {"test": True}):
                comt = "DynamoDB table {0} is set to be deleted".format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(boto_dynamodb.absent(name), ret)

            changes = {
                "new": "Table new_table deleted",
                "old": "Table new_table exists",
            }

            with patch.dict(boto_dynamodb.__opts__, {"test": False}):
                comt = "Deleted DynamoDB table {0}".format(name)
                ret.update({"comment": comt, "result": True, "changes": changes})
                self.assertDictEqual(boto_dynamodb.absent(name), ret)



# -*- coding: utf-8 -*-
"""
    :synopsis: Unit Tests for Windows SNMP Module 'module.win_snmp'
    :platform: Windows
    :maturity: develop
    .. versionadded:: 2017.7.0
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.win_snmp as win_snmp
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase

COMMUNITY_NAMES = {"TestCommunity": "Read Create"}


class WinSnmpTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.win_snmp
    """

    def setup_loader_modules(self):
        return {win_snmp: {}}

    def test_get_agent_service_types(self):
        """
        Test - Get the sysServices types that can be configured.
        """
        self.assertIsInstance(win_snmp.get_agent_service_types(), list)

    def test_get_permission_types(self):
        """
        Test - Get the permission types that can be configured for communities.
        """
        self.assertIsInstance(win_snmp.get_permission_types(), list)

    def test_get_auth_traps_enabled(self):
        """
        Test - Determine whether the host is configured to send authentication traps.
        """
        mock_value = MagicMock(return_value={"vdata": 1})
        with patch.dict(win_snmp.__utils__, {"reg.read_value": mock_value}):
            self.assertTrue(win_snmp.get_auth_traps_enabled())

    def test_set_auth_traps_enabled(self):
        """
        Test - Manage the sending of authentication traps.
        """
        mock_value = MagicMock(return_value=True)
        kwargs = {"status": True}
        with patch.dict(win_snmp.__utils__, {"reg.set_value": mock_value}), patch(
            "salt.modules.win_snmp.get_auth_traps_enabled", MagicMock(return_value=True)
        ):
            self.assertTrue(win_snmp.set_auth_traps_enabled(**kwargs))

    def test_get_community_names(self):
        """
        Test - Get the current accepted SNMP community names and their permissions.
        """
        mock_ret = MagicMock(return_value=[{"vdata": 16, "vname": "TestCommunity"}])
        mock_false = MagicMock(return_value=False)
        with patch.dict(
            win_snmp.__utils__,
            {"reg.list_values": mock_ret, "reg.key_exists": mock_false},
        ):
            self.assertEqual(win_snmp.get_community_names(), COMMUNITY_NAMES)

    def test_get_community_names_gpo(self):
        """
        Test - Get the current accepted SNMP community names and their permissions.
        """
        mock_ret = MagicMock(return_value=[{"vdata": "TestCommunity", "vname": 1}])
        mock_false = MagicMock(return_value=True)
        with patch.dict(
            win_snmp.__utils__,
            {"reg.list_values": mock_ret, "reg.key_exists": mock_false},
        ):
            self.assertEqual(
                win_snmp.get_community_names(), {"TestCommunity": "Managed by GPO"}
            )

    def test_set_community_names(self):
        """
        Test - Manage the SNMP accepted community names and their permissions.
        """
        mock_true = MagicMock(return_value=True)
        kwargs = {"communities": COMMUNITY_NAMES}
        mock_false = MagicMock(return_value=False)
        with patch.dict(
            win_snmp.__utils__,
            {"reg.set_value": mock_true, "reg.key_exists": mock_false},
        ), patch(
            "salt.modules.win_snmp.get_community_names",
            MagicMock(return_value=COMMUNITY_NAMES),
        ):
            self.assertTrue(win_snmp.set_community_names(**kwargs))

    def test_set_community_names_gpo(self):
        """
        Test - Manage the SNMP accepted community names and their permissions.
        """
        mock_true = MagicMock(return_value=True)
        kwargs = {"communities": COMMUNITY_NAMES}
        with patch.dict(
            win_snmp.__utils__,
            {"reg.set_value": mock_true, "reg.key_exists": mock_true},
        ), patch(
            "salt.modules.win_snmp.get_community_names",
            MagicMock(return_value=COMMUNITY_NAMES),
        ):
            self.assertRaises(
                CommandExecutionError, win_snmp.set_community_names, **kwargs
            )



# -*- coding: utf-8 -*-
"""
Tests for salt.states.zpool

:codeauthor:    Jorge Schrauwen <sjorge@blackdot.be>
:maintainer:    Jorge Schrauwen <sjorge@blackdot.be>
:maturity:      new
:depends:       salt.utils.zfs, salt.modules.zpool
:platform:      illumos,freebsd,linux
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Utils
import salt.loader
import salt.states.zpool as zpool

# Import Salt Execution module to test
import salt.utils.zfs
from salt.utils.odict import OrderedDict

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase

# Import test data from salt.utils.zfs test
from tests.unit.utils.test_zfs import ZFSMockData

utils_patch = ZFSMockData().get_patched_utils()


class ZpoolTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.zpool
    """

    def setup_loader_modules(self):
        self.opts = opts = salt.config.DEFAULT_MINION_OPTS.copy()
        utils = salt.loader.utils(opts, whitelist=["zfs"])
        zpool_obj = {
            zpool: {
                "__opts__": opts,
                "__grains__": {"kernel": "SunOS"},
                "__utils__": utils,
            }
        }

        return zpool_obj

    def test_absent_without_pool(self):
        """
        Test zpool absent without a pool
        """
        ret = {
            "name": "myzpool",
            "result": True,
            "comment": "storage pool myzpool is absent",
            "changes": {},
        }

        mock_exists = MagicMock(return_value=False)
        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}), patch.dict(
            zpool.__utils__, utils_patch
        ):
            self.assertEqual(zpool.absent("myzpool"), ret)

    def test_absent_destroy_pool(self):
        """
        Test zpool absent destroying pool
        """
        ret = {
            "name": "myzpool",
            "result": True,
            "comment": "storage pool myzpool was destroyed",
            "changes": {"myzpool": "destroyed"},
        }

        mock_exists = MagicMock(return_value=True)
        mock_destroy = MagicMock(return_value=OrderedDict([("destroyed", True)]))
        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}), patch.dict(
            zpool.__salt__, {"zpool.destroy": mock_destroy}
        ), patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(zpool.absent("myzpool"), ret)

    def test_absent_exporty_pool(self):
        """
        Test zpool absent exporting pool
        """
        ret = {
            "name": "myzpool",
            "result": True,
            "comment": "storage pool myzpool was exported",
            "changes": {"myzpool": "exported"},
        }

        mock_exists = MagicMock(return_value=True)
        mock_destroy = MagicMock(return_value=OrderedDict([("exported", True)]))
        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}), patch.dict(
            zpool.__salt__, {"zpool.export": mock_destroy}
        ), patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(zpool.absent("myzpool", export=True), ret)

    def test_absent_busy(self):
        """
        Test zpool absent on a busy pool
        """
        ret = {
            "name": "myzpool",
            "result": False,
            "comment": "\n".join(
                [
                    "cannot unmount '/myzpool': Device busy",
                    "cannot export 'myzpool': pool is busy",
                ]
            ),
            "changes": {},
        }

        mock_exists = MagicMock(return_value=True)
        mock_destroy = MagicMock(
            return_value=OrderedDict(
                [
                    ("exported", False),
                    (
                        "error",
                        "\n".join(
                            [
                                "cannot unmount '/myzpool': Device busy",
                                "cannot export 'myzpool': pool is busy",
                            ]
                        ),
                    ),
                ]
            )
        )
        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}), patch.dict(
            zpool.__salt__, {"zpool.export": mock_destroy}
        ), patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(zpool.absent("myzpool", export=True), ret)

    def test_present_import_success(self):
        """
        Test zpool present with import allowed and unimported pool
        """
        ret = {
            "name": "myzpool",
            "result": True,
            "comment": "storage pool myzpool was imported",
            "changes": {"myzpool": "imported"},
        }

        config = {
            "import": True,
        }

        mock_exists = MagicMock(return_value=False)
        mock_import = MagicMock(return_value=OrderedDict([("imported", True)]))
        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}), patch.dict(
            zpool.__salt__, {"zpool.import": mock_import}
        ), patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(zpool.present("myzpool", config=config), ret)

    def test_present_import_fail(self):
        """
        Test zpool present with import allowed and no unimported pool or layout
        """
        ret = {
            "name": "myzpool",
            "result": False,
            "comment": "storage pool myzpool was not imported, no (valid) layout specified for creation",
            "changes": {},
        }

        config = {
            "import": True,
        }

        mock_exists = MagicMock(return_value=False)
        mock_import = MagicMock(return_value=OrderedDict([("imported", False)]))
        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}), patch.dict(
            zpool.__salt__, {"zpool.import": mock_import}
        ), patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(zpool.present("myzpool", config=config), ret)

    def test_present_create_success(self):
        """
        Test zpool present with non existing pool
        """
        ret = {
            "name": "myzpool",
            "result": True,
            "comment": "storage pool myzpool was created",
            "changes": {"myzpool": "created"},
        }

        config = {
            "import": False,
        }
        layout = [
            OrderedDict([("mirror", ["disk0", "disk1"])]),
            OrderedDict([("mirror", ["disk2", "disk3"])]),
        ]
        properties = {
            "autoexpand": True,
        }
        filesystem_properties = {
            "quota": "5G",
        }

        mock_exists = MagicMock(return_value=False)
        mock_create = MagicMock(
            return_value=OrderedDict(
                [
                    ("created", True),
                    (
                        "vdevs",
                        OrderedDict(
                            [
                                ("mirror-0", ["/dev/dsk/disk0", "/dev/dsk/disk1"]),
                                ("mirror-1", ["/dev/dsk/disk2", "/dev/dsk/disk3"]),
                            ]
                        ),
                    ),
                ]
            )
        )
        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}), patch.dict(
            zpool.__salt__, {"zpool.create": mock_create}
        ), patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(
                zpool.present(
                    "myzpool",
                    config=config,
                    layout=layout,
                    properties=properties,
                    filesystem_properties=filesystem_properties,
                ),
                ret,
            )

    def test_present_create_fail(self):
        """
        Test zpool present with non existing pool (without a layout)
        """
        ret = {
            "name": "myzpool",
            "result": False,
            "comment": "storage pool myzpool was not imported, no (valid) layout specified for creation",
            "changes": {},
        }

        config = {
            "import": False,
        }

        mock_exists = MagicMock(return_value=False)
        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}), patch.dict(
            zpool.__utils__, utils_patch
        ):
            self.assertEqual(zpool.present("myzpool", config=config), ret)

    def test_present_create_passthrough_fail(self):
        """
        Test zpool present with non existing pool (without a layout)
        """
        ret = {
            "name": "myzpool",
            "result": False,
            "comment": "\n".join(
                [
                    "invalid vdev specification",
                    "use 'force=True' to override the following errors:",
                    "/data/salt/vdisk0 is part of exported pool 'zsalt'",
                    "/data/salt/vdisk1 is part of exported pool 'zsalt'",
                ]
            ),
            "changes": {},
        }

        config = {
            "force": False,
            "import": False,
        }
        layout = [
            OrderedDict([("mirror", ["disk0", "disk1"])]),
            OrderedDict([("mirror", ["disk2", "disk3"])]),
        ]
        properties = {
            "autoexpand": True,
        }
        filesystem_properties = {
            "quota": "5G",
        }

        mock_exists = MagicMock(return_value=False)
        mock_create = MagicMock(
            return_value=OrderedDict(
                [
                    ("created", False),
                    (
                        "error",
                        "\n".join(
                            [
                                "invalid vdev specification",
                                "use 'force=True' to override the following errors:",
                                "/data/salt/vdisk0 is part of exported pool 'zsalt'",
                                "/data/salt/vdisk1 is part of exported pool 'zsalt'",
                            ]
                        ),
                    ),
                ]
            )
        )
        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}), patch.dict(
            zpool.__salt__, {"zpool.create": mock_create}
        ), patch.dict(zpool.__utils__, utils_patch):
            self.assertEqual(
                zpool.present(
                    "myzpool",
                    config=config,
                    layout=layout,
                    properties=properties,
                    filesystem_properties=filesystem_properties,
                ),
                ret,
            )

    def test_present_update_success(self):
        """
        Test zpool present with an existing pool that needs an update
        """
        ret = {
            "name": "myzpool",
            "result": True,
            "comment": "properties updated",
            "changes": {"myzpool": {"autoexpand": False}},
        }

        config = {
            "import": False,
        }
        layout = [
            OrderedDict([("mirror", ["disk0", "disk1"])]),
            OrderedDict([("mirror", ["disk2", "disk3"])]),
        ]
        properties = {
            "autoexpand": False,
        }

        mock_exists = MagicMock(return_value=True)
        mock_get = MagicMock(
            return_value=OrderedDict(
                [
                    ("comment", "salt managed pool"),
                    ("freeing", 0),
                    ("listsnapshots", False),
                    ("leaked", 0),
                    ("feature@obsolete_counts", "enabled"),
                    ("feature@sha512", "enabled"),
                    ("delegation", True),
                    ("dedupditto", "0"),
                    ("dedupratio", "1.00x"),
                    ("autoexpand", True),
                    ("feature@bookmarks", "enabled"),
                    ("allocated", 115712),
                    ("guid", 1591906802560842214),
                    ("feature@large_blocks", "enabled"),
                    ("size", 2113929216),
                    ("feature@enabled_txg", "active"),
                    ("feature@hole_birth", "active"),
                    ("capacity", 0),
                    ("feature@multi_vdev_crash_dump", "enabled"),
                    ("feature@extensible_dataset", "enabled"),
                    ("cachefile", "-"),
                    ("bootfs", "-"),
                    ("autoreplace", True),
                    ("readonly", False),
                    ("version", "-"),
                    ("health", "ONLINE"),
                    ("expandsize", "-"),
                    ("feature@embedded_data", "active"),
                    ("feature@lz4_compress", "active"),
                    ("feature@async_destroy", "enabled"),
                    ("feature@skein", "enabled"),
                    ("feature@empty_bpobj", "enabled"),
                    ("feature@spacemap_histogram", "active"),
                    ("bootsize", "-"),
                    ("free", 2113813504),
                    ("feature@device_removal", "enabled"),
                    ("failmode", "wait"),
                    ("feature@filesystem_limits", "enabled"),
                    ("feature@edonr", "enabled"),
                    ("altroot", "-"),
                    ("fragmentation", "0%"),
                ]
            )
        )
        mock_set = MagicMock(return_value=OrderedDict([("set", True)]))
        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}), patch.dict(
            zpool.__salt__, {"zpool.get": mock_get}
        ), patch.dict(zpool.__salt__, {"zpool.set": mock_set}), patch.dict(
            zpool.__utils__, utils_patch
        ):
            self.assertEqual(
                zpool.present(
                    "myzpool", config=config, layout=layout, properties=properties,
                ),
                ret,
            )

    def test_present_update_nochange_success(self):
        """
        Test zpool present with non existing pool
        """
        config = {
            "import": False,
        }
        layout = [
            OrderedDict([("mirror", ["disk0", "disk1"])]),
            OrderedDict([("mirror", ["disk2", "disk3"])]),
        ]
        properties = {
            "autoexpand": True,
        }

        mock_exists = MagicMock(return_value=True)
        mock_get = MagicMock(
            return_value=OrderedDict(
                [
                    ("comment", "salt managed pool"),
                    ("freeing", 0),
                    ("listsnapshots", False),
                    ("leaked", 0),
                    ("feature@obsolete_counts", "enabled"),
                    ("feature@sha512", "enabled"),
                    ("delegation", True),
                    ("dedupditto", "0"),
                    ("dedupratio", "1.00x"),
                    ("autoexpand", True),
                    ("feature@bookmarks", "enabled"),
                    ("allocated", 115712),
                    ("guid", 1591906802560842214),
                    ("feature@large_blocks", "enabled"),
                    ("size", 2113929216),
                    ("feature@enabled_txg", "active"),
                    ("feature@hole_birth", "active"),
                    ("capacity", 0),
                    ("feature@multi_vdev_crash_dump", "enabled"),
                    ("feature@extensible_dataset", "enabled"),
                    ("cachefile", "-"),
                    ("bootfs", "-"),
                    ("autoreplace", True),
                    ("readonly", False),
                    ("version", "-"),
                    ("health", "ONLINE"),
                    ("expandsize", "-"),
                    ("feature@embedded_data", "active"),
                    ("feature@lz4_compress", "active"),
                    ("feature@async_destroy", "enabled"),
                    ("feature@skein", "enabled"),
                    ("feature@empty_bpobj", "enabled"),
                    ("feature@spacemap_histogram", "active"),
                    ("bootsize", "-"),
                    ("free", 2113813504),
                    ("feature@device_removal", "enabled"),
                    ("failmode", "wait"),
                    ("feature@filesystem_limits", "enabled"),
                    ("feature@edonr", "enabled"),
                    ("altroot", "-"),
                    ("fragmentation", "0%"),
                ]
            )
        )

        ret = {
            "name": "myzpool",
            "result": True,
            "comment": "no update needed",
            "changes": {},
        }

        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}):
            with patch.dict(zpool.__salt__, {"zpool.get": mock_get}):
                with patch.dict(zpool.__utils__, utils_patch):
                    self.assertEqual(
                        zpool.present(
                            "myzpool",
                            config=config,
                            layout=layout,
                            properties=properties,
                        ),
                        ret,
                    )

        # Run state with test=true
        ret = {
            "name": "myzpool",
            "result": True,
            "comment": "storage pool myzpool is uptodate",
            "changes": {},
        }

        with patch.dict(zpool.__salt__, {"zpool.exists": mock_exists}):
            with patch.dict(zpool.__salt__, {"zpool.get": mock_get}):
                with patch.dict(zpool.__utils__, utils_patch):
                    with patch.dict(zpool.__opts__, {"test": True}):
                        self.assertEqual(
                            zpool.present(
                                "myzpool",
                                config=config,
                                layout=layout,
                                properties=properties,
                            ),
                            ret,
                        )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Rupesh Tare <rupesht@saltstack.com>
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import sys

# Import Salt Libs
import salt.modules.genesis as genesis

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class GenesisTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.genesis
    """

    def setup_loader_modules(self):
        return {genesis: {}}

    def test_bootstrap(self):
        """
        Test for Create an image for a specific platform.
        """
        # Changed in 3.7.0 pformat no longer includes the comma
        if sys.version_info >= (3, 7):
            exception_string = "Exception({0})".format(repr("foo"))
        else:
            exception_string = "Exception({0},)".format(repr("foo"))
        mock = MagicMock(return_value=False)
        with patch.dict(genesis.__salt__, {"file.directory_exists": mock}):
            mock = MagicMock(side_effect=Exception("foo"))
            with patch.dict(genesis.__salt__, {"file.mkdir": mock}):
                self.assertEqual(
                    genesis.bootstrap("platform", "root"), {"Error": exception_string}
                )

        with patch.object(genesis, "_bootstrap_yum", return_value="A"):
            with patch.dict(
                genesis.__salt__,
                {
                    "mount.umount": MagicMock(),
                    "file.rmdir": MagicMock(),
                    "file.directory_exists": MagicMock(),
                },
            ):
                with patch.dict(
                    genesis.__salt__, {"disk.blkid": MagicMock(return_value={})}
                ):
                    self.assertEqual(genesis.bootstrap("rpm", "root", "dir"), None)

        common_parms = {
            "platform": "deb",
            "root": "root",
            "img_format": "dir",
            "arch": "amd64",
            "flavor": "stable",
            "static_qemu": "qemu",
        }

        param_sets = [
            {
                "params": {},
                "cmd": [
                    "debootstrap",
                    "--foreign",
                    "--arch",
                    "amd64",
                    "stable",
                    "root",
                    "http://ftp.debian.org/debian/",
                ],
            },
            {
                "params": {"pkgs": "vim"},
                "cmd": [
                    "debootstrap",
                    "--foreign",
                    "--arch",
                    "amd64",
                    "--include",
                    "vim",
                    "stable",
                    "root",
                    "http://ftp.debian.org/debian/",
                ],
            },
            {
                "params": {"pkgs": "vim,emacs"},
                "cmd": [
                    "debootstrap",
                    "--foreign",
                    "--arch",
                    "amd64",
                    "--include",
                    "vim,emacs",
                    "stable",
                    "root",
                    "http://ftp.debian.org/debian/",
                ],
            },
            {
                "params": {"pkgs": ["vim", "emacs"]},
                "cmd": [
                    "debootstrap",
                    "--foreign",
                    "--arch",
                    "amd64",
                    "--include",
                    "vim,emacs",
                    "stable",
                    "root",
                    "http://ftp.debian.org/debian/",
                ],
            },
            {
                "params": {"pkgs": ["vim", "emacs"], "exclude_pkgs": ["vim", "foo"]},
                "cmd": [
                    "debootstrap",
                    "--foreign",
                    "--arch",
                    "amd64",
                    "--include",
                    "vim,emacs",
                    "--exclude",
                    "vim,foo",
                    "stable",
                    "root",
                    "http://ftp.debian.org/debian/",
                ],
            },
        ]

        for param_set in param_sets:

            with patch.dict(
                genesis.__salt__,
                {
                    "mount.umount": MagicMock(),
                    "file.rmdir": MagicMock(),
                    "file.directory_exists": MagicMock(),
                    "cmd.run": MagicMock(),
                    "disk.blkid": MagicMock(return_value={}),
                },
            ):
                with patch(
                    "salt.modules.genesis.salt.utils.path.which", return_value=True
                ):
                    with patch(
                        "salt.modules.genesis.salt.utils.validate.path.is_executable",
                        return_value=True,
                    ):
                        param_set["params"].update(common_parms)
                        self.assertEqual(genesis.bootstrap(**param_set["params"]), None)
                        genesis.__salt__["cmd.run"].assert_any_call(
                            param_set["cmd"], python_shell=False
                        )

        with patch.object(
            genesis, "_bootstrap_pacman", return_value="A"
        ) as pacman_patch:
            with patch.dict(
                genesis.__salt__,
                {
                    "mount.umount": MagicMock(),
                    "file.rmdir": MagicMock(),
                    "file.directory_exists": MagicMock(),
                    "disk.blkid": MagicMock(return_value={}),
                },
            ):
                genesis.bootstrap("pacman", "root", "dir")
                pacman_patch.assert_called_with(
                    "root", img_format="dir", exclude_pkgs=[], pkgs=[]
                )

    def test_avail_platforms(self):
        """
        Test for Return which platforms are available
        """
        with patch("salt.utils.path.which", MagicMock(return_value=False)):
            self.assertFalse(genesis.avail_platforms()["deb"])

    def test_pack(self):
        """
        Test for Pack up a directory structure, into a specific format
        """
        with patch.object(genesis, "_tar", return_value="tar"):
            self.assertEqual(genesis.pack("name", "root"), None)

    def test_unpack(self):
        """
        Test for Unpack an image into a directory structure
        """
        with patch.object(genesis, "_untar", return_value="untar"):
            self.assertEqual(genesis.unpack("name", "root"), None)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.xapi_virt as xapi

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, mock_open, patch
from tests.support.unit import TestCase


class Mockxapi(object):
    """
        Mock xapi class
    """

    def __init__(self):
        pass

    class Session(object):
        """
            Mock Session class
        """

        def __init__(self, xapi_uri):
            pass

        class xenapi(object):
            """
                Mock xenapi class
            """

            def __init__(self):
                pass

            @staticmethod
            def login_with_password(xapi_login, xapi_password):
                """
                    Mock login_with_password method
                """
                return xapi_login, xapi_password

            class session(object):
                """
                    Mock session class
                """

                def __init__(self):
                    pass

                @staticmethod
                def logout():
                    """
                        Mock logout method
                    """
                    return Mockxapi()


class XapiTestCase(TestCase, LoaderModuleMockMixin):
    """
        Test cases for salt.modules.xapi
    """

    def setup_loader_modules(self):
        return {xapi: {}}

    def test_list_domains(self):
        """
            Test to return a list of domain names on the minion
        """
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            self.assertListEqual(xapi.list_domains(), [])

    def test_vm_info(self):
        """
            Test to return detailed information about the vms
        """
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            mock = MagicMock(return_value=False)
            with patch.object(xapi, "_get_record_by_label", mock):
                self.assertDictEqual(xapi.vm_info(True), {True: False})

    def test_vm_state(self):
        """
            Test to return list of all the vms and their state.
        """
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            mock = MagicMock(return_value={"power_state": "1"})
            with patch.object(xapi, "_get_record_by_label", mock):
                self.assertDictEqual(xapi.vm_state("salt"), {"salt": "1"})

                self.assertDictEqual(xapi.vm_state(), {})

    def test_get_nics(self):
        """
            Test to return info about the network interfaces of a named vm
        """
        ret = {"Stack": {"device": "ETH0", "mac": "Stack", "mtu": 1}}
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            mock = MagicMock(side_effect=[False, {"VIFs": "salt"}])
            with patch.object(xapi, "_get_record_by_label", mock):
                self.assertFalse(xapi.get_nics("salt"))

                mock = MagicMock(
                    return_value={"MAC": "Stack", "device": "ETH0", "MTU": 1}
                )
                with patch.object(xapi, "_get_record", mock):
                    self.assertDictEqual(xapi.get_nics("salt"), ret)

    def test_get_macs(self):
        """
            Test to return a list off MAC addresses from the named vm
        """
        mock = MagicMock(side_effect=[None, ["a", "b", "c"]])
        with patch.object(xapi, "get_nics", mock):
            self.assertIsNone(xapi.get_macs("salt"))

            self.assertListEqual(xapi.get_macs("salt"), ["a", "b", "c"])

    def test_get_disks(self):
        """
            Test to return the disks of a named vm
        """
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            mock = MagicMock(side_effect=[False, ["a", "b", "c"]])
            with patch.object(xapi, "_get_label_uuid", mock):
                self.assertFalse(xapi.get_disks("salt"))

                self.assertDictEqual(xapi.get_disks("salt"), {})

    def test_setmem(self):
        """
            Test to changes the amount of memory allocated to VM.
        """
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            mock = MagicMock(side_effect=[False, ["a", "b", "c"]])
            with patch.object(xapi, "_get_label_uuid", mock):
                self.assertFalse(xapi.setmem("salt", "1"))

                self.assertTrue(xapi.setmem("salt", "1"))

        with patch.object(xapi, "_check_xenapi", MagicMock(return_value=Mockxapi)):
            mock = MagicMock(return_value=True)
            with patch.dict(xapi.__salt__, {"config.option": mock}):
                with patch.object(xapi, "_get_label_uuid", mock):
                    self.assertFalse(xapi.setmem("salt", "1"))

    def test_setvcpus(self):
        """
            Test to changes the amount of vcpus allocated to VM.
        """
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            mock = MagicMock(side_effect=[False, ["a", "b", "c"]])
            with patch.object(xapi, "_get_label_uuid", mock):
                self.assertFalse(xapi.setvcpus("salt", "1"))

                self.assertTrue(xapi.setvcpus("salt", "1"))

        with patch.object(xapi, "_check_xenapi", MagicMock(return_value=Mockxapi)):
            mock = MagicMock(return_value=True)
            with patch.dict(xapi.__salt__, {"config.option": mock}):
                with patch.object(xapi, "_get_label_uuid", mock):
                    self.assertFalse(xapi.setvcpus("salt", "1"))

    def test_vcpu_pin(self):
        """
            Test to Set which CPUs a VCPU can use.
        """
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            mock = MagicMock(side_effect=[False, ["a", "b", "c"]])
            with patch.object(xapi, "_get_label_uuid", mock):
                self.assertFalse(xapi.vcpu_pin("salt", "1", "2"))

                self.assertTrue(xapi.vcpu_pin("salt", "1", "2"))

        with patch.object(xapi, "_check_xenapi", MagicMock(return_value=Mockxapi)):
            mock = MagicMock(return_value=True)
            with patch.dict(xapi.__salt__, {"config.option": mock}):
                with patch.object(xapi, "_get_label_uuid", mock):
                    with patch.dict(xapi.__salt__, {"cmd.run": mock}):
                        self.assertTrue(xapi.vcpu_pin("salt", "1", "2"))

    def test_freemem(self):
        """
            Test to return an int representing the amount of memory
            that has not been given to virtual machines on this node
        """
        mock = MagicMock(return_value={"free_memory": 1024})
        with patch.object(xapi, "node_info", mock):
            self.assertEqual(xapi.freemem(), 1024)

    def test_freecpu(self):
        """
            Test to return an int representing the number
            of unallocated cpus on this hypervisor
        """
        mock = MagicMock(return_value={"free_cpus": 1024})
        with patch.object(xapi, "node_info", mock):
            self.assertEqual(xapi.freecpu(), 1024)

    def test_full_info(self):
        """
            Test to return the node_info, vm_info and freemem
        """
        mock = MagicMock(return_value="salt")
        with patch.object(xapi, "node_info", mock):
            mock = MagicMock(return_value="stack")
            with patch.object(xapi, "vm_info", mock):
                self.assertDictEqual(
                    xapi.full_info(), {"node_info": "salt", "vm_info": "stack"}
                )

    def test_shutdown(self):
        """
            Test to send a soft shutdown signal to the named vm
        """
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            mock = MagicMock(side_effect=[False, ["a", "b", "c"]])
            with patch.object(xapi, "_get_label_uuid", mock):
                self.assertFalse(xapi.shutdown("salt"))

                self.assertTrue(xapi.shutdown("salt"))

        with patch.object(xapi, "_check_xenapi", MagicMock(return_value=Mockxapi)):
            mock = MagicMock(return_value=True)
            with patch.dict(xapi.__salt__, {"config.option": mock}):
                with patch.object(xapi, "_get_label_uuid", mock):
                    self.assertFalse(xapi.shutdown("salt"))

    def test_pause(self):
        """
            Test to pause the named vm
        """
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            mock = MagicMock(side_effect=[False, ["a", "b", "c"]])
            with patch.object(xapi, "_get_label_uuid", mock):
                self.assertFalse(xapi.pause("salt"))

                self.assertTrue(xapi.pause("salt"))

        with patch.object(xapi, "_check_xenapi", MagicMock(return_value=Mockxapi)):
            mock = MagicMock(return_value=True)
            with patch.dict(xapi.__salt__, {"config.option": mock}):
                with patch.object(xapi, "_get_label_uuid", mock):
                    self.assertFalse(xapi.pause("salt"))

    def test_resume(self):
        """
            Test to resume the named vm
        """
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            mock = MagicMock(side_effect=[False, ["a", "b", "c"]])
            with patch.object(xapi, "_get_label_uuid", mock):
                self.assertFalse(xapi.resume("salt"))

                self.assertTrue(xapi.resume("salt"))

        with patch.object(xapi, "_check_xenapi", MagicMock(return_value=Mockxapi)):
            mock = MagicMock(return_value=True)
            with patch.dict(xapi.__salt__, {"config.option": mock}):
                with patch.object(xapi, "_get_label_uuid", mock):
                    self.assertFalse(xapi.resume("salt"))

    def test_start(self):
        """
            Test to reboot a domain via ACPI request
        """
        mock = MagicMock(return_value=True)
        with patch.object(xapi, "start", mock):
            self.assertTrue(xapi.start("salt"))

    def test_reboot(self):
        """
            Test to reboot a domain via ACPI request
        """
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            mock = MagicMock(side_effect=[False, ["a", "b", "c"]])
            with patch.object(xapi, "_get_label_uuid", mock):
                self.assertFalse(xapi.reboot("salt"))

                self.assertTrue(xapi.reboot("salt"))

        with patch.object(xapi, "_check_xenapi", MagicMock(return_value=Mockxapi)):
            mock = MagicMock(return_value=True)
            with patch.dict(xapi.__salt__, {"config.option": mock}):
                with patch.object(xapi, "_get_label_uuid", mock):
                    self.assertFalse(xapi.reboot("salt"))

    def test_reset(self):
        """
            Test to reset a VM by emulating the
            reset button on a physical machine
        """
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            mock = MagicMock(side_effect=[False, ["a", "b", "c"]])
            with patch.object(xapi, "_get_label_uuid", mock):
                self.assertFalse(xapi.reset("salt"))

                self.assertTrue(xapi.reset("salt"))

        with patch.object(xapi, "_check_xenapi", MagicMock(return_value=Mockxapi)):
            mock = MagicMock(return_value=True)
            with patch.dict(xapi.__salt__, {"config.option": mock}):
                with patch.object(xapi, "_get_label_uuid", mock):
                    self.assertFalse(xapi.reset("salt"))

    def test_migrate(self):
        """
            Test to migrates the virtual machine to another hypervisor
        """
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            mock = MagicMock(side_effect=[False, ["a", "b", "c"]])
            with patch.object(xapi, "_get_label_uuid", mock):
                self.assertFalse(xapi.migrate("salt", "stack"))

                self.assertTrue(xapi.migrate("salt", "stack"))

        with patch.object(xapi, "_check_xenapi", MagicMock(return_value=Mockxapi)):
            mock = MagicMock(return_value=True)
            with patch.dict(xapi.__salt__, {"config.option": mock}):
                with patch.object(xapi, "_get_label_uuid", mock):
                    self.assertFalse(xapi.migrate("salt", "stack"))

    def test_stop(self):
        """
            Test to Hard power down the virtual machine,
            this is equivalent to pulling the power
        """
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            mock = MagicMock(side_effect=[False, ["a", "b", "c"]])
            with patch.object(xapi, "_get_label_uuid", mock):
                self.assertFalse(xapi.stop("salt"))

                self.assertTrue(xapi.stop("salt"))

        with patch.object(xapi, "_check_xenapi", MagicMock(return_value=Mockxapi)):
            mock = MagicMock(return_value=True)
            with patch.dict(xapi.__salt__, {"config.option": mock}):
                with patch.object(xapi, "_get_label_uuid", mock):
                    self.assertFalse(xapi.stop("salt"))

    def test_is_hyper(self):
        """
            Test to returns a bool whether or not
            this node is a hypervisor of any kind
        """
        with patch.dict(xapi.__grains__, {"virtual_subtype": "Dom0"}):
            self.assertFalse(xapi.is_hyper())

        with patch.dict(xapi.__grains__, {"virtual": "Xen Dom0"}):
            self.assertFalse(xapi.is_hyper())

        with patch.dict(xapi.__grains__, {"virtual_subtype": "Xen Dom0"}):
            with patch("salt.utils.files.fopen", mock_open(read_data="salt")):
                self.assertFalse(xapi.is_hyper())

            with patch("salt.utils.files.fopen", mock_open()) as mock_read:
                mock_read.side_effect = IOError
                self.assertFalse(xapi.is_hyper())

            with patch("salt.utils.files.fopen", mock_open(read_data="xen_")):
                with patch.dict(xapi.__grains__, {"ps": "salt"}):
                    mock = MagicMock(return_value={"xenstore": "salt"})
                    with patch.dict(xapi.__salt__, {"cmd.run": mock}):
                        self.assertTrue(xapi.is_hyper())

    def test_vm_cputime(self):
        """
            Test to Return cputime used by the vms
        """
        ret = {"1": {"cputime_percent": 0, "cputime": 1}}
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            mock = MagicMock(return_value={"host_CPUs": "1"})
            with patch.object(xapi, "_get_record_by_label", mock):
                mock = MagicMock(
                    return_value={"VCPUs_number": "1", "VCPUs_utilisation": {"0": "1"}}
                )
                with patch.object(xapi, "_get_metrics_record", mock):
                    self.assertDictEqual(xapi.vm_cputime("1"), ret)

            mock = MagicMock(return_value={})
            with patch.object(xapi, "list_domains", mock):
                self.assertDictEqual(xapi.vm_cputime(""), {})

    def test_vm_netstats(self):
        """
            Test to return combined network counters used by the vms
        """
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            self.assertDictEqual(xapi.vm_netstats(""), {})

    def test_vm_diskstats(self):
        """
            Test to return disk usage counters used by the vms
        """
        with patch.object(xapi, "_get_xapi_session", MagicMock()):
            self.assertDictEqual(xapi.vm_diskstats(""), {})



# -*- coding: utf-8 -*-
"""
Integration tests for the saltutil module.
"""

from __future__ import absolute_import, print_function, unicode_literals

import os
import shutil
import textwrap
import threading
import time

import pytest
import salt.config
import salt.defaults.events
import salt.utils.event
import salt.utils.files
import salt.utils.stringutils
from tests.support.case import ModuleCase
from tests.support.helpers import flaky, slowTest
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import skipIf


@pytest.mark.windows_whitelisted
@pytest.mark.usefixtures("salt_sub_minion")
class SaltUtilModuleTest(ModuleCase):
    """
    Testcase for the saltutil execution module
    """

    @classmethod
    def setUpClass(cls):
        # Whell functions, on a minion, must run with the master running
        # along side the minion.
        # We copy the master config to the minion's configuration directory just
        # for this test since the test suite master and minion(s) do not share the
        # same configuration directory
        src = os.path.join(RUNTIME_VARS.TMP_CONF_DIR, "master")
        dst = os.path.join(RUNTIME_VARS.TMP_MINION_CONF_DIR, "master")
        cls.copied_master_config_file = None
        if not os.path.exists(dst):
            shutil.copyfile(src, dst)
            cls.copied_master_config_file = dst

    @classmethod
    def tearDownClass(cls):
        if cls.copied_master_config_file and os.path.exists(
            cls.copied_master_config_file
        ):
            os.unlink(cls.copied_master_config_file)
        cls.copied_master_config_file = None

    def setUp(self):
        self.run_function("saltutil.refresh_pillar")

    # Tests for the wheel function

    @slowTest
    def test_wheel_just_function(self):
        """
        Tests using the saltutil.wheel function when passing only a function.
        """
        # Wait for the pillar refresh to kick in, so that grains are ready to go
        time.sleep(3)
        ret = self.run_function("saltutil.wheel", ["minions.connected"])
        self.assertIn("minion", ret["return"])
        self.assertIn("sub_minion", ret["return"])

    @slowTest
    def test_wheel_with_arg(self):
        """
        Tests using the saltutil.wheel function when passing a function and an arg.
        """
        ret = self.run_function("saltutil.wheel", ["key.list", "minion"])
        self.assertEqual(ret["return"], {})

    @slowTest
    def test_wheel_no_arg_raise_error(self):
        """
        Tests using the saltutil.wheel function when passing a function that requires
        an arg, but one isn't supplied.
        """
        self.assertRaises(TypeError, "saltutil.wheel", ["key.list"])

    @slowTest
    def test_wheel_with_kwarg(self):
        """
        Tests using the saltutil.wheel function when passing a function and a kwarg.
        This function just generates a key pair, but doesn't do anything with it. We
        just need this for testing purposes.
        """
        ret = self.run_function("saltutil.wheel", ["key.gen"], keysize=1024)
        self.assertIn("pub", ret["return"])
        self.assertIn("priv", ret["return"])


@pytest.mark.windows_whitelisted
class SyncGrainsTest(ModuleCase):
    @slowTest
    def test_sync_grains(self):
        ret = self.run_function("saltutil.sync_grains")
        self.assertEqual(ret, [])


@pytest.mark.windows_whitelisted
class SaltUtilSyncModuleTest(ModuleCase):
    """
    Testcase for the saltutil sync execution module
    """

    def setUp(self):
        whitelist = {
            "modules": [],
        }
        self.run_function("saltutil.sync_all", extmod_whitelist=whitelist)

    def tearDown(self):
        self.run_function("saltutil.sync_all")

    @slowTest
    def test_sync_all(self):
        """
        Test syncing all ModuleCase
        """
        expected_return = {
            "engines": [],
            "clouds": [],
            "grains": [],
            "beacons": [],
            "utils": [],
            "returners": [],
            "modules": [
                "modules.depends_versioned",
                "modules.depends_versionless",
                "modules.mantest",
                "modules.override_test",
                "modules.runtests_decorators",
                "modules.runtests_helpers",
                "modules.salttest",
            ],
            "renderers": [],
            "log_handlers": [],
            "matchers": [],
            "states": [],
            "sdb": [],
            "proxymodules": [],
            "executors": [],
            "output": [],
            "thorium": [],
            "serializers": [],
        }
        ret = self.run_function("saltutil.sync_all")
        self.assertEqual(ret, expected_return)

    @slowTest
    def test_sync_all_whitelist(self):
        """
        Test syncing all ModuleCase with whitelist
        """
        expected_return = {
            "engines": [],
            "clouds": [],
            "grains": [],
            "beacons": [],
            "utils": [],
            "returners": [],
            "modules": ["modules.salttest"],
            "renderers": [],
            "log_handlers": [],
            "matchers": [],
            "states": [],
            "sdb": [],
            "proxymodules": [],
            "executors": [],
            "output": [],
            "thorium": [],
            "serializers": [],
        }
        ret = self.run_function(
            "saltutil.sync_all", extmod_whitelist={"modules": ["salttest"]}
        )
        self.assertEqual(ret, expected_return)

    @slowTest
    def test_sync_all_blacklist(self):
        """
        Test syncing all ModuleCase with blacklist
        """
        expected_return = {
            "engines": [],
            "clouds": [],
            "grains": [],
            "beacons": [],
            "utils": [],
            "returners": [],
            "modules": [
                "modules.mantest",
                "modules.override_test",
                "modules.runtests_helpers",
                "modules.salttest",
            ],
            "renderers": [],
            "log_handlers": [],
            "matchers": [],
            "states": [],
            "sdb": [],
            "proxymodules": [],
            "executors": [],
            "output": [],
            "thorium": [],
            "serializers": [],
        }
        ret = self.run_function(
            "saltutil.sync_all",
            extmod_blacklist={
                "modules": [
                    "runtests_decorators",
                    "depends_versioned",
                    "depends_versionless",
                ]
            },
        )
        self.assertEqual(ret, expected_return)

    @slowTest
    def test_sync_all_blacklist_and_whitelist(self):
        """
        Test syncing all ModuleCase with whitelist and blacklist
        """
        expected_return = {
            "engines": [],
            "clouds": [],
            "grains": [],
            "beacons": [],
            "utils": [],
            "returners": [],
            "executors": [],
            "modules": [],
            "renderers": [],
            "log_handlers": [],
            "matchers": [],
            "states": [],
            "sdb": [],
            "proxymodules": [],
            "output": [],
            "thorium": [],
            "serializers": [],
        }
        ret = self.run_function(
            "saltutil.sync_all",
            extmod_whitelist={"modules": ["runtests_decorators"]},
            extmod_blacklist={"modules": ["runtests_decorators"]},
        )
        self.assertEqual(ret, expected_return)


@skipIf(True, "Pillar refresh test is flaky. Skipping for now.")
@pytest.mark.windows_whitelisted
class SaltUtilSyncPillarTest(ModuleCase):
    """
    Testcase for the saltutil sync pillar module
    """

    class WaitForEvent(threading.Thread):
        def __init__(self, opts, event_tag):
            self.__eventer = salt.utils.event.get_event(
                "minion", opts=opts, listen=True
            )
            self.__event_tag = event_tag
            self.__event_complete = False

            threading.Thread.__init__(self)

        def run(self):
            if self.__eventer.get_event(tag=self.__event_tag, wait=30):
                self.__event_complete = True

        def is_complete(self):
            return self.__event_complete

    @flaky
    def test_pillar_refresh(self):
        """
        test pillar refresh module
        """
        pillar_key = "itworked"

        pre_pillar = self.run_function("pillar.raw")
        self.assertNotIn(pillar_key, pre_pillar.get(pillar_key, "didnotwork"))

        with salt.utils.files.fopen(
            os.path.join(RUNTIME_VARS.TMP_PILLAR_TREE, "add_pillar.sls"), "w"
        ) as fp:
            fp.write(salt.utils.stringutils.to_str("{0}: itworked".format(pillar_key)))

        with salt.utils.files.fopen(
            os.path.join(RUNTIME_VARS.TMP_PILLAR_TREE, "top.sls"), "w"
        ) as fp:
            fp.write(
                textwrap.dedent(
                    """\
                     base:
                       '*':
                         - add_pillar
                     """
                )
            )

        self.run_function("saltutil.refresh_pillar")

        pillar = False
        timeout = time.time() + 30
        while not pillar:
            post_pillar = self.run_function("pillar.raw")
            try:
                self.assertIn(pillar_key, post_pillar.get(pillar_key, "didnotwork"))
                pillar = True
            except AssertionError:
                if time.time() > timeout:
                    self.assertIn(pillar_key, post_pillar.get(pillar_key, "didnotwork"))
                continue

        post_pillar = self.run_function("pillar.raw")
        self.assertIn(pillar_key, post_pillar.get(pillar_key, "didnotwork"))

    def test_pillar_refresh_sync(self):
        """
        test pillar refresh module with sync enabled
        """
        pillar_key = "itworked_sync"

        pre_pillar = self.run_function("pillar.raw")
        self.assertNotIn(pillar_key, pre_pillar.get(pillar_key, "didnotwork_sync"))

        with salt.utils.files.fopen(
            os.path.join(RUNTIME_VARS.TMP_PILLAR_TREE, "add_pillar_sync.sls"), "w"
        ) as fp:
            fp.write(
                salt.utils.stringutils.to_str("{0}: itworked_sync".format(pillar_key))
            )

        with salt.utils.files.fopen(
            os.path.join(RUNTIME_VARS.TMP_PILLAR_TREE, "top.sls"), "w"
        ) as fp:
            fp.write(
                textwrap.dedent(
                    """\
                     base:
                       '*':
                         - add_pillar_sync
                     """
                )
            )

        opts = self.run_function("test.get_opts")
        wait = self.WaitForEvent(
            opts, salt.defaults.events.MINION_PILLAR_REFRESH_COMPLETE
        )
        wait.start()
        self.run_function("saltutil.refresh_pillar", wait=True)
        while wait.is_alive():
            time.sleep(1)
        self.assertTrue(wait.is_complete())

        pillar = False
        timeout = time.time() + 30
        while not pillar:
            post_pillar = self.run_function("pillar.raw")
            try:
                self.assertIn(
                    pillar_key, post_pillar.get(pillar_key, "didnotwork_sync")
                )
                pillar = True
            except AssertionError:
                if time.time() > timeout:
                    self.assertIn(
                        pillar_key, post_pillar.get(pillar_key, "didnotwork_sync")
                    )
                continue

        post_pillar = self.run_function("pillar.raw")
        self.assertIn(pillar_key, post_pillar.get(pillar_key, "didnotwork_sync"))

    def tearDown(self):
        for filename in os.listdir(RUNTIME_VARS.TMP_PILLAR_TREE):
            os.remove(os.path.join(RUNTIME_VARS.TMP_PILLAR_TREE, filename))



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import os

# Import salt libs
import salt.modules.linux_service as service

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class ServiceTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.linux_service
    """

    def setup_loader_modules(self):
        return {service: {}}

    def test_start(self):
        """
        Test to start the specified service
        """
        with patch.object(os.path, "join", return_value="A"):
            with patch.object(service, "run", MagicMock(return_value=True)):
                self.assertTrue(service.start("name"))

    def test_stop(self):
        """
        Test to stop the specified service
        """
        with patch.object(os.path, "join", return_value="A"):
            with patch.object(service, "run", MagicMock(return_value=True)):
                self.assertTrue(service.stop("name"))

    def test_restart(self):
        """
        Test to restart the specified service
        """
        with patch.object(os.path, "join", return_value="A"):
            with patch.object(service, "run", MagicMock(return_value=True)):
                self.assertTrue(service.restart("name"))

    def test_status(self):
        """
        Test to return the status for a service, returns the PID or an empty
        string if the service is running or not, pass a signature to use to
        find the service via ps
        """
        with patch.dict(service.__salt__, {"status.pid": MagicMock(return_value=True)}):
            self.assertTrue(service.status("name"))

    def test_reload_(self):
        """
        Test to restart the specified service
        """
        with patch.object(os.path, "join", return_value="A"):
            with patch.object(service, "run", MagicMock(return_value=True)):
                self.assertTrue(service.reload_("name"))

    def test_run(self):
        """
        Test to run the specified service
        """
        with patch.object(os.path, "join", return_value="A"):
            with patch.object(service, "run", MagicMock(return_value=True)):
                self.assertTrue(service.run("name", "action"))

    def test_available(self):
        """
        Test to returns ``True`` if the specified service is available,
        otherwise returns ``False``.
        """
        with patch.object(service, "get_all", return_value=["name", "A"]):
            self.assertTrue(service.available("name"))

    def test_missing(self):
        """
        Test to inverse of service.available.
        """
        with patch.object(service, "get_all", return_value=["name1", "A"]):
            self.assertTrue(service.missing("name"))

    def test_get_all(self):
        """
        Test to return a list of all available services
        """
        with patch.object(os.path, "isdir", side_effect=[False, True]):

            self.assertEqual(service.get_all(), [])

            with patch.object(os, "listdir", return_value=["A", "B"]):
                self.assertListEqual(service.get_all(), ["A", "B"])



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.supervisord as supervisord

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class SupervisordTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.supervisord
    """

    def setup_loader_modules(self):
        return {supervisord: {}}

    # 'running' function tests: 1

    def test_running(self):
        """
        Test to ensure the named service is running.
        """
        name = "wsgi_server"

        ret = {"name": name, "changes": {}, "result": True, "comment": ""}

        comt = (
            "Supervisord module not activated." " Do you need to install supervisord?"
        )
        ret.update({"comment": comt, "result": False})
        self.assertDictEqual(supervisord.running(name), ret)

        mock = MagicMock(return_value={name: {"state": "running"}})
        with patch.dict(supervisord.__salt__, {"supervisord.status": mock}):
            with patch.dict(supervisord.__opts__, {"test": True}):
                comt = "Service wsgi_server is already running"
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(supervisord.running(name), ret)

            with patch.dict(supervisord.__opts__, {"test": False}):
                comt = "Not starting already running service: wsgi_server"
                ret.update({"comment": comt})
                self.assertDictEqual(supervisord.running(name), ret)

    # 'dead' function tests: 1

    def test_dead(self):
        """
        Test to ensure the named service is dead (not running).
        """
        name = "wsgi_server"

        ret = {"name": name, "changes": {}, "result": None, "comment": ""}

        with patch.dict(supervisord.__opts__, {"test": True}):
            comt = "Service {0} is set to be stopped".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(supervisord.dead(name), ret)

    # 'mod_watch' function tests: 1

    def test_mod_watch(self):
        """
        Test to always restart on watch.
        """
        name = "wsgi_server"

        ret = {"name": name, "changes": {}, "result": None, "comment": ""}

        comt = (
            "Supervisord module not activated." " Do you need to install supervisord?"
        )
        ret.update({"comment": comt, "result": False})
        self.assertDictEqual(supervisord.mod_watch(name), ret)



# -*- coding: utf-8 -*-

"""
tests for user state
user absent
user present
user present with custom homedir
"""

from __future__ import absolute_import, print_function, unicode_literals

import os
import sys
from random import randint

import pytest
import salt.utils.files
import salt.utils.platform
from tests.support.case import ModuleCase
from tests.support.helpers import (
    destructiveTest,
    requires_system_grains,
    skip_if_not_root,
    slowTest,
)
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.unit import skipIf

try:
    import grp
except ImportError:
    grp = None

if salt.utils.platform.is_darwin():
    USER = "macuser"
    GROUP = "macuser"
    GID = randint(400, 500)
    NOGROUPGID = randint(400, 500)
elif salt.utils.platform.is_windows():
    USER = "winuser"
    GROUP = "winuser"
    GID = randint(400, 500)
    NOGROUPGID = randint(400, 500)
else:
    USER = "nobody"
    GROUP = "nobody"
    GID = "nobody"
    NOGROUPGID = "nogroup"


@destructiveTest
@skip_if_not_root
@pytest.mark.windows_whitelisted
class UserTest(ModuleCase, SaltReturnAssertsMixin):
    """
    test for user absent
    """

    user_name = "salt-test"
    alt_group = "salt-test-altgroup"
    user_home = (
        "/var/lib/{0}".format(user_name)
        if not salt.utils.platform.is_windows()
        else os.path.join("tmp", user_name)
    )

    @slowTest
    def test_user_absent(self):
        ret = self.run_state("user.absent", name="unpossible")
        self.assertSaltTrueReturn(ret)

    @slowTest
    def test_user_if_present(self):
        ret = self.run_state("user.present", name=USER)
        self.assertSaltTrueReturn(ret)

    @slowTest
    def test_user_if_present_with_gid(self):
        if self.run_function("group.info", [USER]):
            ret = self.run_state("user.present", name=USER, gid=GID)
        elif self.run_function("group.info", ["nogroup"]):
            ret = self.run_state("user.present", name=USER, gid=NOGROUPGID)
        else:
            self.skipTest("Neither 'nobody' nor 'nogroup' are valid groups")
        self.assertSaltTrueReturn(ret)

    @slowTest
    def test_user_not_present(self):
        """
        This is a DESTRUCTIVE TEST it creates a new user on the minion.
        And then destroys that user.
        Assume that it will break any system you run it on.
        """
        ret = self.run_state("user.present", name=self.user_name)
        self.assertSaltTrueReturn(ret)

    @slowTest
    def test_user_present_when_home_dir_does_not_18843(self):
        """
        This is a DESTRUCTIVE TEST it creates a new user on the minion.
        And then destroys that user.
        Assume that it will break any system you run it on.
        """
        if salt.utils.platform.is_darwin():
            HOMEDIR = "/Users/home_of_" + self.user_name
        else:
            HOMEDIR = "/home/home_of_" + self.user_name
        ret = self.run_state("user.present", name=self.user_name, home=HOMEDIR)
        self.assertSaltTrueReturn(ret)

        self.run_function("file.absent", name=HOMEDIR)
        ret = self.run_state("user.present", name=self.user_name, home=HOMEDIR)
        self.assertSaltTrueReturn(ret)

    @requires_system_grains
    @slowTest
    def test_user_present_nondefault(self, grains=None):
        """
        This is a DESTRUCTIVE TEST it creates a new user on the on the minion.
        """
        ret = self.run_state("user.present", name=self.user_name, home=self.user_home)
        self.assertSaltTrueReturn(ret)
        ret = self.run_function("user.info", [self.user_name])
        self.assertReturnNonEmptySaltType(ret)
        if salt.utils.platform.is_windows():
            group_name = self.run_function("user.list_groups", [self.user_name])
        else:
            group_name = grp.getgrgid(ret["gid"]).gr_name

        if not salt.utils.platform.is_darwin() and not salt.utils.platform.is_windows():
            self.assertTrue(os.path.isdir(self.user_home))
        if grains["os_family"] in ("Suse",):
            self.assertEqual(group_name, "users")
        elif grains["os_family"] == "MacOS":
            self.assertEqual(group_name, "staff")
        elif salt.utils.platform.is_windows():
            self.assertEqual([], group_name)
        else:
            self.assertEqual(group_name, self.user_name)

    @skipIf(
        salt.utils.platform.is_windows(), "windows minion does not support usergroup"
    )
    @slowTest
    def test_user_present_usergroup_false(self):
        """
        This is a DESTRUCTIVE TEST it creates a new user on the on the minion.
        This is a unit test, NOT an integration test. We create a group of the
        same name as the user beforehand, so it should all run smoothly.
        """
        # MacOS users' primary group defaults to staff (20), not the name of
        # user
        ret = self.run_state("group.present", name=self.user_name)
        self.assertSaltTrueReturn(ret)
        if salt.utils.platform.is_darwin():
            gid = grp.getgrnam("staff").gr_gid
        else:
            gid = self.user_name
        ret = self.run_state(
            "user.present",
            name=self.user_name,
            gid=gid,
            usergroup=False,
            home=self.user_home,
        )
        self.assertSaltTrueReturn(ret)
        ret = self.run_function("user.info", [self.user_name])
        self.assertReturnNonEmptySaltType(ret)
        group_name = grp.getgrgid(ret["gid"]).gr_name
        if not salt.utils.platform.is_darwin():
            self.assertTrue(os.path.isdir(self.user_home))
            self.assertEqual(group_name, self.user_name)
        ret = self.run_state("user.absent", name=self.user_name)
        self.assertSaltTrueReturn(ret)
        ret = self.run_state("group.absent", name=self.user_name)
        self.assertSaltTrueReturn(ret)

    @skipIf(
        salt.utils.platform.is_windows(),
        "windows minion does not support gid_from_name",
    )
    @skipIf(
        salt.utils.platform.is_windows(), "windows minion does not support usergroup"
    )
    def test_user_present_usergroup(self):
        """
        This is a DESTRUCTIVE TEST it creates a new user on the on the minion.
        This is a unit test, NOT an integration test.
        """
        ret = self.run_state(
            "user.present", name=self.user_name, usergroup=True, home=self.user_home
        )
        self.assertSaltTrueReturn(ret)

        ret = self.run_function("user.info", [self.user_name])
        self.assertReturnNonEmptySaltType(ret)
        group_name = grp.getgrgid(ret["gid"]).gr_name

        if not salt.utils.platform.is_darwin():
            self.assertTrue(os.path.isdir(self.user_home))
        if salt.utils.platform.is_darwin():
            group_name = "staff"
        else:
            group_name = self.user_name
        self.assertEqual(group_name, group_name)
        ret = self.run_state("user.absent", name=self.user_name)
        self.assertSaltTrueReturn(ret)
        if not salt.utils.platform.is_darwin():
            ret = self.run_state("group.absent", name=self.user_name)
            self.assertSaltTrueReturn(ret)

    @skipIf(
        sys.getfilesystemencoding().startswith("ANSI"),
        "A system encoding which supports Unicode characters must be set. Current setting is: {0}. Try setting $LANG='en_US.UTF-8'".format(
            sys.getfilesystemencoding()
        ),
    )
    @slowTest
    def test_user_present_unicode(self):
        """
        This is a DESTRUCTIVE TEST it creates a new user on the on the minion.

        It ensures that unicode GECOS data will be properly handled, without
        any encoding-related failures.
        """
        ret = self.run_state(
            "user.present",
            name=self.user_name,
            fullname="Sålt Test",
            roomnumber="①②③",
            workphone="١٢٣٤",
            homephone="६७८",
        )
        self.assertSaltTrueReturn(ret)
        # Ensure updating a user also works
        ret = self.run_state(
            "user.present",
            name=self.user_name,
            fullname="Sølt Test",
            roomnumber="①③②",
            workphone="٣٤١٢",
            homephone="६८७",
        )
        self.assertSaltTrueReturn(ret)

    @skipIf(
        salt.utils.platform.is_windows(),
        "windows minion does not support roomnumber or phone",
    )
    @slowTest
    def test_user_present_gecos(self):
        """
        This is a DESTRUCTIVE TEST it creates a new user on the on the minion.

        It ensures that numeric GECOS data will be properly coerced to strings,
        otherwise the state will fail because the GECOS fields are written as
        strings (and show up in the user.info output as such). Thus the
        comparison will fail, since '12345' != 12345.
        """
        ret = self.run_state(
            "user.present",
            name=self.user_name,
            fullname=12345,
            roomnumber=123,
            workphone=1234567890,
            homephone=1234567890,
        )
        self.assertSaltTrueReturn(ret)

    @skipIf(
        salt.utils.platform.is_windows(),
        "windows minion does not support roomnumber or phone",
    )
    @slowTest
    def test_user_present_gecos_none_fields(self):
        """
        This is a DESTRUCTIVE TEST it creates a new user on the on the minion.

        It ensures that if no GECOS data is supplied, the fields will be coerced
        into empty strings as opposed to the string "None".
        """
        ret = self.run_state(
            "user.present",
            name=self.user_name,
            fullname=None,
            roomnumber=None,
            workphone=None,
            homephone=None,
        )
        self.assertSaltTrueReturn(ret)

        ret = self.run_function("user.info", [self.user_name])
        self.assertReturnNonEmptySaltType(ret)
        self.assertEqual("", ret["fullname"])
        # MacOS does not supply the following GECOS fields
        if not salt.utils.platform.is_darwin():
            self.assertEqual("", ret["roomnumber"])
            self.assertEqual("", ret["workphone"])
            self.assertEqual("", ret["homephone"])

    @skipIf(
        salt.utils.platform.is_windows(), "windows minion does not support createhome"
    )
    @slowTest
    def test_user_present_home_directory_created(self):
        """
        This is a DESTRUCTIVE TEST it creates a new user on the minion.

        It ensures that the home directory is created.
        """
        ret = self.run_state("user.present", name=self.user_name, createhome=True)
        self.assertSaltTrueReturn(ret)

        user_info = self.run_function("user.info", [self.user_name])
        self.assertTrue(os.path.exists(user_info["home"]))

    @skipIf(
        salt.utils.platform.is_windows() or salt.utils.platform.is_darwin(),
        "groups/gid not fully supported",
    )
    def test_user_present_change_gid_but_keep_group(self):
        """
        This tests the case in which the default group is changed at the same
        time as it is also moved into the "groups" list.
        """
        try:
            # Add the groups
            ret = self.run_state("group.present", name=self.user_name)
            self.assertSaltTrueReturn(ret)
            ret = self.run_state("group.present", name=self.alt_group)
            self.assertSaltTrueReturn(ret)

            user_name_gid = self.run_function("file.group_to_gid", [self.user_name])
            alt_group_gid = self.run_function("file.group_to_gid", [self.alt_group])

            # Add the user
            ret = self.run_state("user.present", name=self.user_name, gid=alt_group_gid)

            # Check that the initial user addition set the gid and groups as
            # expected.
            new_gid = self.run_function("file.group_to_gid", [self.user_name])
            uinfo = self.run_function("user.info", [self.user_name])

            assert uinfo["gid"] == alt_group_gid, uinfo["gid"]
            assert uinfo["groups"] == [self.alt_group], uinfo["groups"]

            # Now change the gid and move alt_group to the groups list in the
            # same salt run.
            ret = self.run_state(
                "user.present",
                name=self.user_name,
                gid=user_name_gid,
                groups=[self.alt_group],
                allow_gid_change=True,
            )
            self.assertSaltTrueReturn(ret)

            # Be sure that we did what we intended
            new_gid = self.run_function("file.group_to_gid", [self.user_name])
            uinfo = self.run_function("user.info", [self.user_name])

            assert uinfo["gid"] == new_gid, uinfo["gid"]
            assert uinfo["groups"] == [self.user_name, self.alt_group], uinfo["groups"]

        finally:
            # Do the cleanup here so we don't have to put all of this in the
            # tearDown to be executed after each test.
            self.assertSaltTrueReturn(
                self.run_state("user.absent", name=self.user_name)
            )
            self.assertSaltTrueReturn(
                self.run_state("group.absent", name=self.user_name)
            )
            self.assertSaltTrueReturn(
                self.run_state("group.absent", name=self.alt_group)
            )

    def tearDown(self):
        if salt.utils.platform.is_darwin():
            check_user = self.run_function("user.list_users")
            if USER in check_user:
                del_user = self.run_function("user.delete", [USER], remove=True)
        self.assertSaltTrueReturn(self.run_state("user.absent", name=self.user_name))


@destructiveTest
@skip_if_not_root
@skipIf(not salt.utils.platform.is_windows(), "Windows only tests")
@pytest.mark.windows_whitelisted
class WinUserTest(ModuleCase, SaltReturnAssertsMixin):
    """
    test for user absent
    """

    def tearDown(self):
        self.assertSaltTrueReturn(self.run_state("user.absent", name=USER))

    @slowTest
    def test_user_present_existing(self):
        ret = self.run_state(
            "user.present",
            name=USER,
            win_homedrive="U:",
            win_profile="C:\\User\\{0}".format(USER),
            win_logonscript="C:\\logon.vbs",
            win_description="Test User Account",
        )
        self.assertSaltTrueReturn(ret)
        ret = self.run_state(
            "user.present",
            name=USER,
            win_homedrive="R:",
            win_profile="C:\\Users\\{0}".format(USER),
            win_logonscript="C:\\Windows\\logon.vbs",
            win_description="Temporary Account",
        )
        self.assertSaltTrueReturn(ret)
        self.assertSaltStateChangesEqual(ret, "R:", keys=["homedrive"])
        self.assertSaltStateChangesEqual(
            ret, "C:\\Users\\{0}".format(USER), keys=["profile"]
        )
        self.assertSaltStateChangesEqual(
            ret, "C:\\Windows\\logon.vbs", keys=["logonscript"]
        )
        self.assertSaltStateChangesEqual(ret, "Temporary Account", keys=["description"])



# -*- coding: utf-8 -*-

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt libs
import salt.renderers.toml as toml
import salt.serializers.toml

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf


@skipIf(not salt.serializers.toml.available, "The 'toml' library is missing")
class TOMLRendererTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {toml: {}}

    def test_toml_render_string(self):
        data = """[[user-sshkey."ssh_auth.present"]]
                    user = "username"
                    [[user-sshkey."ssh_auth.present"]]
                    config = "%h/.ssh/authorized_keys"
                    [[user-sshkey."ssh_auth.present"]]
                    names = [
                      "hereismykey",
                      "anotherkey"
                    ]
                """
        expected_result = {
            "user-sshkey": {
                "ssh_auth.present": [
                    {"user": "username"},
                    {"config": "%h/.ssh/authorized_keys"},
                    {"names": ["hereismykey", "anotherkey"]},
                ]
            }
        }
        result = toml.render(data)

        self.assertEqual(result, expected_result)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import salt.modules.serverdensity_device as serverdensity_device

# Import Salt Libs
import salt.utils.json
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class MockRequests(object):
    """
    Mock smtplib class
    """

    flag = None
    content = """{"message": "Invalid token", "errors": [{"type": "invalid_token", "subject": "token"}]}"""
    status_code = None

    def __init__(self):
        self.url = None
        self.data = None
        self.kwargs = None

    def return_request(self, url, data=None, **kwargs):
        """
        Mock request method.
        """
        self.url = url
        self.data = data
        self.kwargs = kwargs
        requests = MockRequests()
        if self.flag == 1:
            requests.status_code = 401
        else:
            requests.status_code = 200
        return requests

    def post(self, url, data=None, **kwargs):
        """
        Mock post method.
        """
        return self.return_request(url, data, **kwargs)

    def delete(self, url, **kwargs):
        """
        Mock delete method.
        """
        return self.return_request(url, **kwargs)

    def get(self, url, **kwargs):
        """
        Mock get method.
        """
        return self.return_request(url, **kwargs)

    def put(self, url, data=None, **kwargs):
        """
        Mock put method.
        """
        return self.return_request(url, data, **kwargs)


class ServerdensityDeviceTestCase(TestCase, LoaderModuleMockMixin):
    """
    TestCase for salt.modules.serverdensity_device
    """

    def setup_loader_modules(self):
        return {serverdensity_device: {"requests": MockRequests()}}

    def setUp(self):
        self.mock_json_loads = MagicMock(side_effect=ValueError())

    # 'get_sd_auth' function tests: 1

    def test_get_sd_auth(self):
        """
        Tests if it returns requested Server Density
        authentication value from pillar.
        """
        with patch.dict(serverdensity_device.__pillar__, {"serverdensity": False}):
            self.assertRaises(
                CommandExecutionError, serverdensity_device.get_sd_auth, "1"
            )

        with patch.dict(
            serverdensity_device.__pillar__, {"serverdensity": {"1": "salt"}}
        ):
            self.assertEqual(serverdensity_device.get_sd_auth("1"), "salt")

            self.assertRaises(
                CommandExecutionError, serverdensity_device.get_sd_auth, "2"
            )

    # 'create' function tests: 1

    def test_create(self):
        """
        Tests if it create device in Server Density.
        """
        with patch.dict(
            serverdensity_device.__pillar__, {"serverdensity": {"api_token": "salt"}}
        ):
            self.assertTrue(serverdensity_device.create("rich_lama", group="lama_band"))

            with patch.object(salt.utils.json, "loads", self.mock_json_loads):
                self.assertRaises(
                    CommandExecutionError,
                    serverdensity_device.create,
                    "rich_lama",
                    group="lama_band",
                )

            MockRequests.flag = 1
            self.assertIsNone(
                serverdensity_device.create("rich_lama", group="lama_band")
            )

    # 'delete' function tests: 1

    def test_delete(self):
        """
        Tests if it delete a device from Server Density.
        """
        with patch.dict(
            serverdensity_device.__pillar__, {"serverdensity": {"api_token": "salt"}}
        ):
            MockRequests.flag = 0
            self.assertTrue(serverdensity_device.delete("51f7eaf"))

            with patch.object(salt.utils.json, "loads", self.mock_json_loads):
                self.assertRaises(
                    CommandExecutionError, serverdensity_device.delete, "51f7eaf"
                )

            MockRequests.flag = 1
            self.assertIsNone(serverdensity_device.delete("51f7eaf"))

    # 'ls' function tests: 1

    def test_ls(self):
        """
        Tests if it list devices in Server Density.
        """
        with patch.dict(
            serverdensity_device.__pillar__, {"serverdensity": {"api_token": "salt"}}
        ):
            MockRequests.flag = 0
            self.assertTrue(serverdensity_device.ls(name="lama"))

            with patch.object(salt.utils.json, "loads", self.mock_json_loads):
                self.assertRaises(
                    CommandExecutionError, serverdensity_device.ls, name="lama"
                )

            MockRequests.flag = 1
            self.assertIsNone(serverdensity_device.ls(name="lama"))

    # 'update' function tests: 1

    def test_update(self):
        """
        Tests if it updates device information in Server Density.
        """
        with patch.dict(
            serverdensity_device.__pillar__, {"serverdensity": {"api_token": "salt"}}
        ):
            MockRequests.flag = 0
            self.assertTrue(serverdensity_device.update("51f7eaf", name="lama"))

            with patch.object(salt.utils.json, "loads", self.mock_json_loads):
                self.assertRaises(
                    CommandExecutionError,
                    serverdensity_device.update,
                    "51f7eaf",
                    name="lama",
                )

            MockRequests.flag = 1
            self.assertIsNone(serverdensity_device.update("51f7eaf", name="lama"))

    # 'install_agent' function tests: 1

    def test_install_agent(self):
        """
        Tests if it downloads Server Density installation agent,
        and installs sd-agent with agent_key.
        """
        mock = MagicMock(return_value=True)
        with patch.dict(
            serverdensity_device.__pillar__, {"serverdensity": {"account_url": "salt"}}
        ):
            with patch.dict(serverdensity_device.__salt__, {"cmd.run": mock}):
                with patch.dict(serverdensity_device.__opts__, {"cachedir": "/"}):
                    self.assertTrue(serverdensity_device.install_agent("51f7e"))

    # 'install_agent_v2' function tests: 1

    def test_install_agent_v2(self):
        """
        Tests if it downloads Server Density installation agent,
        and installs sd-agent with agent_key.
        """
        mock = MagicMock(return_value=True)
        with patch.dict(
            serverdensity_device.__pillar__, {"serverdensity": {"account_name": "salt"}}
        ):
            with patch.dict(serverdensity_device.__salt__, {"cmd.run": mock}):
                with patch.dict(serverdensity_device.__opts__, {"cachedir": "/"}):
                    self.assertTrue(
                        serverdensity_device.install_agent("51f7e", agent_version=2)
                    )
